# bgo.tcl---BGO card setup

#    March 29 1996  conversion for TK   VP
# Wed Jan 25 17:30:37 GMT 1995, DB
#   substituted $env(EXPBASE) for /eg
# Wed Nov 30 11:02:18 GMT 1994, DB
#   printing via print templates introduced
# 9-12-92 PHO
#	make global action apply to first action only
# 26-10-92 SCL
#	added missing print-pair routine (copied from ge.tcl)
# 23-10-92 PHO
#	put step on sliders [lindex table(xxx) 10]
# 13-10-92 PHO
#	put help on 'By Parameter' frame
# 9-9-92 PHO
#	included feedback after writing register
# 20 Aug 92 PHO
#    implemented help widget to call manual browser
# 11 Aug 92 PHO
#    created new widgets for testmode registers
#    added maxchannel variable
# Fri Jul 10 20:10:25 BST 1992 DB
#    merged in code for 'By Channel' and 'By Parameter' formats

set debug 0
set started 0

set frame BGO
set frametitle "Tapered Detector (BGO) Card setup"
set icontitle BGO

set chanlist {???}
set chanindex 0

set defaultchanlist {0 1 2 3 4 5 6 7}

set cardtype BGO
set VXIGlobal ""

#    The array "table" which is indexed by the "widget name" contains all the information needed
#    for the register/display item
#
#    Each element of "table" is a list
#
#       0.    name of global parameter used to hold current value of the register
#       1.    procedure to be used to convert the widget value into parameter value
#       2.    procedure to be used to convert the parameter value into a widget value
#       3.    name of the register
#       4.    procedure to be used to convert the register value into parameter value
#       5.    procedure to be used to convert the parameter value into a register value
#       6.    name of unit used for register access
#       7.    widget type to be used for "by parameter" displays
#       8-10  parameters for the procedure creating "by parameter" displays
#              eg  if (7) is :slider then 8 is min value, 9 is max value and 10 is unit
#       11    for startup only - initial value of the parameter (1)
#       12    for startup only - inserted for this widget into the table "parameternames"


set table(CHANNEL) {channel widget->channel channel->widget nil
        nil nil nil nil nil nil nil 1 nil}
set table(ALLCHANS) {allchans int-widget->param param->int-widget nil
        nil nil nil nil nil nil nil 0 nil}

set table(ENABLE) {enable int-widget->param param->int-widget "Disable" 
   not-reg->param param->not-reg nil :checkbox nil nil nil 1 "Enable"}
set table(CONFENETRES) {confenetres int-widget->param param->int-widget "ConFenetres" 
     int-reg->param param->int-reg nil :checkbox nil nil nil 1 "Con Fenetres"}
set table(LASTCARD) {lastcard int-widget->param param->int-widget "LastCard" 
   int-reg->param param->int-reg nil :checkbox nil nil nil 1 "Last Card"}
set table(PATTERNTHRESH) {patternthresh int-widget->param param->int-widget "Pattern.Thresh" 
   int-reg->param param->int-reg nil :slider 0 65536 1 1 "Pattern Threshold"}
set table(PATTERNADDR) {patternaddr str-widget->param param->hex-widget "Pattern.Addr" 
   int-reg->param param->int-reg nil :text 6 nil nil 1 "Pattern Address"}
set table(PATTERNSS) {patternss int-widget->param param->int-widget "Pattern.SSEnable" 
   int-reg->param param->int-reg nil :checkbox nil nil nil 1 "Pattern Sliding Scale"}
set table(PATTERNRO) {patternro int-widget->param param->int-widget "Pattern.ROEnable" 
   int-reg->param param->int-reg nil :checkbox nil nil nil 1 "Pattern Readout"}
set table(TACTHRESH) {tacthresh int-widget->param param->int-widget "TAC.Thresh" 
   int-reg->param param->int-reg nil :slider 0 65536 1 1 "TAC Threshold"}
set table(TACADDR) {tacaddr str-widget->param param->hex-widget "TAC.Addr" 
   int-reg->param param->int-reg nil :text 6 nil nil 1 "TAC Address"}
set table(TACSS) {tacss int-widget->param param->int-widget "TAC.SSEnable" 
   int-reg->param param->int-reg nil :checkbox nil nil nil 1 "TAC Sliding Scale"}
set table(TACRO) {tacro int-widget->param param->int-widget "TAC.ROEnable" 
   int-reg->param param->int-reg nil :checkbox nil nil nil 1 "TAC Readout"}
set table(ENERGYTHRESH) {energythresh int-widget->param param->int-widget "Energy.Thresh" 
   int-reg->param param->int-reg nil :slider 0 65536 1 1 "Energy Threshold"}
set table(ENERGYADDR) {energyaddr str-widget->param param->hex-widget "Energy.Addr" 
   int-reg->param param->int-reg nil :text 6 nil nil 1 "Energy Address"}
set table(ENERGYSS) {energyss int-widget->param param->int-widget "Energy.SSEnable" 
   int-reg->param param->int-reg nil :checkbox nil nil nil 1 "Energy Sliding Scale"}
set table(ENERGYRO) {energyro int-widget->param param->int-widget "Energy.ROEnable" 
   int-reg->param param->int-reg nil :checkbox nil nil nil 1 "Energy Readout"}
set table(VETOWIDTH) {vetowidth slider-widget->param param->int-widget "Veto.Width" 
   unit-reg->param param->unit-reg ns :slider 100 1000 4 1 "Veto Width (ns)"}
set table(DTWIDTH) {dtwidth slider-widget->param param->int-widget "DT.Width" 
   unit-reg->param param->unit-reg ns :slider 2000 20000 71 1 "Dead Time Width (ns)"}
set table(SUMWIDTH) {sumwidth slider-widget->param param->int-widget "Sum.Width" 
   unit-reg->param param->unit-reg ns :slider 100 1000 4 1 "Sum Width (ns)"}
set table(FTSAMPLE) {ftsample slider-widget->param param->int-widget "FTSample" 
   unit-reg->param param->unit-reg ns :slider 0 2000 8 1 "Fast Trigger Sample (ns)"}
set table(PDSGATE) {pdsgate slider-widget->param param->int-widget "PDSGate" 
   unit-reg->param param->unit-reg ns :slider 0 10000 40 1 "PDS Gate (ns)"}
set table(WATCHDOG) {watchdog slider-widget->param param->int-widget "Watchdog" 
   unit-reg->param param->unit-reg ns :slider 0 16000 63 1 "LT Watchdog Timeout (ns)"}
set table(VALSAMPLE) {valsample slider-widget->param param->int-widget "ValSample" 
   unit-reg->param param->unit-reg ns :slider 0 10000 40 1 "Validation Sample (ns)"}
set table(DISCTHRESH) {discthresh slider-widget->param param->int-widget "Disc.Thresh" 
   unit-reg->param param->unit-reg keV :slider 0 90 1 1 "Disc Threshold (KeV)"}
set table(TESTGEN) {testgen slider-widget->param param->int-widget "TestGen" 
   unit-reg->param param->unit-reg mV :slider 0 5000 3 1 "Test Generator (mV)"}
foreach i {A B C D E F G H I J} {
  set table(TEST$i) "test$i int-widget->param param->int-widget \"$i.TestMode.Disable\" 
    not-reg->param param->not-reg nil :checkbox nil nil nil 1 nil"
}

set table(MODELCODE) {modcode str-widget->param param->hex-widget "VXI.ModelCode" 
   int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(SERIALNUM) {serialnum str-widget->param param->hex-widget "VXI.SerialNum" 
   int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(MODLEVEL) {modlevel str-widget->param param->hex-widget "VXI.ModLevel" 
   int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(MANID) {manid str-widget->param param->hex-widget "VXI.ManID" 
   int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(OFFSET) {offset str-widget->param param->hex-widget "VXI.Offset" 
   int-reg->param param->int-reg nil nil nil nil nil 1 nil}


#    register selection for parameter save function

set registersavepatterns {
   Disable ConFenetres FTSample ValSample Disc.Thresh
   Veto.Width DT.Width Sum.Width PDSGate
   Pattern.Thresh Pattern.SSEnable Pattern.ROEnable 
   TAC.Thresh     TAC.SSEnable     TAC.ROEnable
   Energy.Thresh  Energy.SSEnable  Energy.ROEnable
}


# inspection tables must be in same order as entries in the :choice-stacks

set  analogueinsptable \
  {"Disconnected" "a.Input" "b.Input" "c.Input" "d.Input" "e.Input" 
   "f.Input" "g.Input" "h.Input" "i.Input" "j.Input" 
   "a.LineRec" "b.LineRec" "c.LineRec" "d.LineRec" "e.LineRec" "f.LineRec" 
   "g.LineRec" "h.LineRec" "i.LineRec" "j.LineRec" 
   "ShFilter" "PeakDet" "TAC" "TestGen"}

set digitalinsptable \
  {"Disconnected" "a.Disc" "b.Disc" "c.Disc" "d.Disc" "e.Disc" "f.Disc" 
   "g.Disc" "h.Disc" "i.Disc" "j.Disc" 
   "ORDisc" "FT" "FTSample" "Val2" "ValSample" "ValAck" "PDSGate" "EncCom"
   "DataAck" "RenIn" "RenOut" "LTReset" "DTDisc" "Veto" "Inhibit" "SumIMult"
   "TACStart" "LT3"}

set voltageinsptable \
  {"Disconnected" "DiscThresh" "SumWidth"  "VetoWidth"  "FTWidth" "ValSample" 
   "DTWidth" "PDSGate" "ROWidth"
   "V-24"  "V-12"  "V-6"  "V-5.2"  "V-2" "V+3"  
   "V+5DAC"  "V+5"  "V+6"  "V+12"  "V+24" "TestGen"}

set insp_table(MUX-AI1) {"Insp.A1" AI}
set insp_table(MUX-AI2) {"Insp.A2" AI}
set insp_table(MUX-DI1) {"Insp.D1" DI}
set insp_table(MUX-DI2) {"Insp.D2" DI}
set insp_table(MUX-VI)  {"Insp.V"  VI}

set insp_menu(MUX-AI1) analogueinsptable
set insp_menu(MUX-AI2) analogueinsptable
set insp_menu(MUX-DI1) digitalinsptable
set insp_menu(MUX-DI2) digitalinsptable
set insp_menu(MUX-VI)  voltageinsptable

set inspection_widgetnames {MUX-AI1 MUX-AI2 MUX-DI1 MUX-DI2 MUX-VI}
set inspection_regnames    {Insp.A1 Insp.A2 Insp.D1 Insp.D2 Insp.V}

#     these are common
set steplist1 "DTWIDTH VETOWIDTH SUMWIDTH FTSAMPLE VALSAMPLE WATCHDOG DISCTHRESH PDSGATE"
#     no per segment items
set steplist2 ""

set register Disable

loadsource common.vxi 

proc click arg {
   global debug userselection allchans
   if {$debug} then {puts stderr "click with argument \{$arg\}"}

   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]
   if {$c==":quit"} then {return [click-quit $f]}
   switch -glob $w {
      REDISPLAY   {click-redisplay}
      ENGINEERING  {click-$v}
      FORMAT      {click-format}
      STEP        {click-step $v $i}

      CHANNEL     {click-channel $v}
      ALLCHANS    {click-whichchans $v}

      PARAM       {click-param $v}
      MUX-*       {click-inspection $w [lindex $arg 4]}

      default   {
          if {$allchans < 0} then {set userselection 1}
             click-on $w $v
             set userselection 0
      }
   }
}


set electronicsframe [self frame-name].electronics

proc make-frame1 {} {
  global env chanlist
  global analogueinsptable digitalinsptable voltageinsptable stepmenu stepmode

  :frame [self frame-name] :label "[self frame-title]" :show-footer T :resize F \
      :icon-image "@$env(ICONHOME)/vxi_module" \
      :icon-name "[self icon-title]"

   :panel P1 :layout T :padx 10 :pady 5
     :layout H; :row-gap 10; :col-gap 10
      :choice-stack CHANNEL :label "Current Channel:" :strings $chanlist :valuex 18c
      :hskip 150
      :checkbox ENABLE :label "Enable:" :valuex 10c
     :next-row T
      :choice ALLCHANS :label "Act on Channels?" :valuex 18c :strings [list Current All Select] :value 0

   :panel P2 :layout T :padx 10
     :layout H; :col-gap 25
      :hskip 75
      :message M1  :label "SS"        :bold T
      :message M2  :label "RO"        :bold T
      :message M3  :label "Address"   :bold T
      :hskip 50
      :message M4  :label "Threshold" :bold T
   :row-gap 5; :next-row T
      :checkbox ENERGYSS      :label "Energy:"  :valuex 12c
      :checkbox ENERGYRO      :label " "
      :text     ENERGYADDR    :label " " :w 6   :read-only T 
      :slider   ENERGYTHRESH  :label " " :min 0 :max 65535
   :next-row T
      :checkbox TACSS         :label "TAC:"     :valuex 12c
      :checkbox TACRO         :label " "
      :text     TACADDR       :label " " :w 6   :read-only T 
      :slider   TACTHRESH     :label " " :min 0 :max 65535
   :next-row T
      :checkbox PATTERNSS     :label "Pattern:" :valuex 12c
      :checkbox PATTERNRO     :label " "
      :text     PATTERNADDR   :label " " :w 6   :read-only T 
      :slider   PATTERNTHRESH :label " " :min 0 :max 65535
   
   :panel P3 :layout T :padx 10 :pady 10
     :layout V; :row-gap 5
      :slider VETOWIDTH  :label "Veto Width (ns):"          :min 0 :max 1000  :valuex 30c
      :slider DTWIDTH    :label "Dead Time Width (ns):"     :min 2000 :max 20000  :valuex 30c
      :slider SUMWIDTH   :label "Sum Width (ns):"           :min 0 :max 1000  :valuex 30c
      :slider FTSAMPLE   :label "Fast Trigger Sample (ns):" :min 0 :max 2000  :valuex 30c
      :slider VALSAMPLE  :label "Validation Sample (ns):"   :min 0 :max 10000 :valuex 30c
      :slider WATCHDOG   :label "LT Watchdog Timeout (ns):" :min 0 :max 16000 :valuex 30c
      :slider PDSGATE    :label "PDS Gate (ns):"            :min 0 :max 10000 :valuex 30c
      :slider DISCTHRESH :label "Disc Threshold (KeV):"     :min 0 :max 90    :valuex 30c
   
      :vskip 10
      :layout H; :col-gap 10; :row-gap 10
      :next-row T
      :menubutton MUX-AI1 :label "Analogue Inspection 1:" :w 20 :menu $analogueinsptable
      :message MUX-AI1-USE   :label ""
      :next-row T
      :menubutton MUX-AI2 :label "Analogue Inspection 2:" :w 20 :menu $analogueinsptable
      :message MUX-AI2-USE   :label ""
      :next-row T
      :menubutton MUX-DI1 :label "Digital Inspection 1:"  :w 20 :menu $digitalinsptable
      :message MUX-DI1-USE   :label ""
      :next-row T
      :menubutton MUX-DI2 :label "Digital Inspection 2:"  :w 20 :menu $digitalinsptable
      :message MUX-DI2-USE   :label ""
      :next-row T
      :menubutton MUX-VI  :label "Voltage Inspection:"    :w 20 :menu $voltageinsptable
      :message MUX-VI-USE   :label ""
   
   :panel P4 :layout B :padx 10 :after FOOTER
     :layout H
      :button REDISPLAY 
      :menubutton STEP      :label "[lindex $stepmenu $stepmode]"  :strings $stepmenu
      :hskip 15
      :button FORMAT :label "By Parameter"
      :menubutton ENGINEERING :label "Engineering"  :menu [list electronics software]
      :button HELP :command click-help
   
 :show T
}

proc make-frame2 {} {
  global env parameternames

  :frame "[self frame-name]" :label "[self frame-title]" :show-footer T  :resize F \
      :icon-image "@$env(ICONHOME)/vxi_module" \
      :icon-name "[self icon-title]"

   :panel P10 :layout T :padx 5 :pady 5
     :layout H 
      :choice-stack PARAM :label "Parameter:" :valuex 15c :strings  $parameternames
      :hskip 200
      :choice ALLCHANS :label "Act on Channels?" :valuex 18c :strings [list Current All Select] :value 0
   
   :panel PANEL :layout T :padx 5

   :panel P11 :layout B :padx 5 :after FOOTER
     :layout H 
      :button REDISPLAY
      :button FORMAT :label "By Channel"
      :hskip 80
      :button HELP   :command click-help
   
   :show T
}

proc make-electronics {} {
  global env electronicsframe electronicsvisible

  :frame "$electronicsframe" :label "BGO Electronics" :resize F :transient T

   :panel P20 :layout T :padx 5 :pady 5
      :layout H; :row-gap 5
      :checkbox LASTCARD  :label "Last Card:"                            :valuex 20c
      :text MODELCODE     :label "Model Code:" :w 6  :read-only T        :valuex 20c
    :next-row T
      :checkbox MODE      :label "Test Mode:"                            :valuex 20c
      :text SERIALNUM     :label "Serial Number:" :w 6 :read-only T      :valuex 20c
    :next-row T
      :checkbox CONFENETRES :label "Controle Fenetres:"                  :valuex 20c
      :text MODLEVEL      :label "Modification Level:" :w 6 :read-only T :valuex 20c
    :next-row T
      :text MANID         :label "Manufacturer ID:" :w 6  :read-only T   :valuex 20c
    :next-row T
      :checkbox ATTN      :label "Test Gen Attenuation:"                 :valuex 20c
      :text OFFSET        :label "Offset Register:" :w 6 :read-only T    :valuex 20c
      :row-gap 20; :next-row T; :row-gap 10
      :choice TESTA    :exclusive F :strings "A" :label "Test Mode:" :w 3
      foreach i {B C D E F G H I J} {
        :choice TEST$i :exclusive F :strings "$i" :w 3
      }
    :next-row T
      :slider TESTGEN     :label "Test Generator (mV):" :min 0 :max 7000  :valuex 20c
   
  :show T
 set electronicsvisible 1
}

proc make-chanSELframe {} {
  global chansel chanlist

   :frame CHANSEL-FRAME :label "BGO Channel Selection"  :show-footer F :resize F :transient T
   :panel CHANSEL-PANEL :layout T
   :choice CHANSEL :exclusive F :strings $chanlist :value $chansel
   :show T
}  
