

#   clusterge.defn.tcl     Euroball III Cluster Ge detector VXI card


proc {AddClusterGeModule}  {module slot}  {
   global registerbase cardtype

   set registerbase $cardtype$module

   delete-vxi-registers $registerbase.*

   ClusterGeVXIRegisters $slot
   ClusterGeModuleRegisters  $slot

   ClusterGeRegisters $module $slot
}

proc ClusterGeRegisters {module slot} {
   global registerbase cardtype

   set registerbase $cardtype$module.Global
   ClusterGeVXIRegisters $slot
   ClusterGeModuleRegisters  $slot

#    registers for SAROC 1 at offset 0x100 GeA => GeD

   set c 0
   foreach segment {A B C D} {
      set registerbase $cardtype$module.Ge$segment
      ClusterGeSAROCRegisters $slot $c 0x100
      incr c
   }

#    registers for SAROC 2 at offset 0x180 GeE => GeH

   set c 0
   foreach segment {E F G H} {
      set registerbase $cardtype$module.Ge$segment
      ClusterGeSAROCRegisters $slot $c 0x180
      incr c
   }

#    channel area registers 

   set c 0
   foreach segment {A B C D E F G H} {
      set registerbase $cardtype$module.Ge$segment
      ClusterGeDetectorRegisters  $slot $c 
      incr c
   }
}

proc clusterge_register {registername class attributes} {
   global registerbase
   define-vxi-register $registerbase $class $registername "$attributes"
}


proc ClusterGeVXIRegisters {slot} {

   clusterge_register  datal          xDig "-s$slot  -o0x0 -w32 -prw -fx"
   clusterge_register  dataw          xDig "-s$slot  -o0x0 -w16 -prw -fx"

   clusterge_register  VXI.ManID      xSht  "-s$slot -o0  -pro -w16 -fx -b0  -l12"
   clusterge_register  VXI.AddrSpace  xSht  "-s$slot -o0  -pro -w16 -fd -b12 -l2"
   clusterge_register  VXI.DevClass   xSht  "-s$slot -o0  -pro -w16 -fd -b14 -l2"
   clusterge_register  VXI.ModelCode  xSht  "-s$slot -o2  -pro -w16 -fx -b0  -l12"
   clusterge_register  VXI.ReqMem     xSht  "-s$slot -o2  -pro -w16 -fd -b12 -l4"
   clusterge_register  VXI.Reset      xSht  "-s$slot -o4  -prw -w16 -fd -b0  -l1"
   clusterge_register  VXI.Passed     xSht  "-s$slot -o4  -pro -w16 -fd -b2  -l1"
   clusterge_register  VXI.Ready      xSht  "-s$slot -o4  -pro -w16 -fd -b3  -l1"
   clusterge_register  VXI.ModID      xSht  "-s$slot -o4  -pro -w16 -fd -b14 -l1"
   clusterge_register  VXI.A24A32     xSht  "-s$slot -o4  -pro -w16 -fd -b15 -l1"
   clusterge_register  VXI.Offset     xSht  "-s$slot -o6  -pro -w16 -fx"
   clusterge_register  VXI.SerialNum  xSht  "-s$slot -o8  -pro -w16 -fx"
   clusterge_register  VXI.ModLevel   xSht  "-s$slot -o10 -pro -w16 -fx"
}


proc ClusterGeModuleRegisters {slot} {
   global registerbase

#    common area registers

	clusterge_register   MCR 	      xDig    "-s$slot  -o0x10 -w16 -prw -fx -b0 -l0    "
	clusterge_register   PileupReject     xDig    "-s$slot  -o0x10 -w16 -prw -fd -b0 -l1 -i0"
	clusterge_register   TFAGain          xDig    "-s$slot  -o0x10 -w16 -prw -fd -b1 -l1 -i1"
	clusterge_register   SSCEnable        xDig    "-s$slot  -o0x10 -w16 -prw -fd -b2 -l1 -i1"
	clusterge_register   SSDACEnable      xDig    "-s$slot  -o0x10 -w16 -prw -fd -b3 -l1 -i0"
	clusterge_register   MCR-b4           xDig    "-s$slot  -o0x10 -w16 -prw -fd -b4 -l1    "
	clusterge_register   PipeLineEnable   xDig    "-s$slot  -o0x10 -w16 -prw -fd -b5 -l1 -i0"
	clusterge_register   EvNumRdOutEnable xDig    "-s$slot  -o0x10 -w16 -prw -fd -b6 -l1 -i0"
	clusterge_register   ReadOutMode      xDig    "-s$slot  -o0x10 -w16 -prw -fd -b7 -l1 -i0"
	clusterge_register   FIFO.Reset       xDig    "-s$slot  -o0x10 -w16 -prw -fd -b8 -l1 -i1"
	clusterge_register   GIR.Reset        xDig    "-s$slot  -o0x10 -w16 -prw -fd -b9 -l1    "
	clusterge_register   LastCard         xDig    "-s$slot  -o0x10 -w16 -prw -fd -b10 -l1    "
	clusterge_register   MCR-b11          xDig    "-s$slot  -o0x10 -w16 -prw -fd -b11 -l1    "
	clusterge_register   MCR-b12          xDig    "-s$slot  -o0x10 -w16 -prw -fd -b12 -l1    "
	clusterge_register   MCR-b14          xDig    "-s$slot  -o0x10 -w16 -prw -fd -b13 -l1    "
	clusterge_register   DSP.Reset        xDig    "-s$slot  -o0x10 -w16 -prw -fd -b14 -l1 -i0"
	clusterge_register   MCR-b15          xDig    "-s$slot  -o0x10 -w16 -prw -fd -b15 -l1    "

	clusterge_register   Insp.D1.Channel   xMux    "-s$slot  -o0x20 -w16 -prw -b0 -l16 -dCluster -md1"
	clusterge_register   Insp.D1.Signal    xDig    "-s$slot  -o0x22 -w16 -prw -fd -b0 -l0 -i0"
	clusterge_register   Insp.D2.Channel   xMux    "-s$slot  -o0x24 -w16 -prw -b0 -l16 -dCluster -md2"
	clusterge_register   Insp.D2.Signal    xDig    "-s$slot  -o0x26 -w16 -prw -fd -b0 -l0 -i0"
	clusterge_register   Insp.A1.Channel   xMux    "-s$slot  -o0x28 -w16 -prw -b0 -l16 -dCluster -ma1"
	clusterge_register   Insp.A1.Signal    xDig    "-s$slot  -o0x2a -w16 -prw -fd -b0 -l0 -i0"
	clusterge_register   Insp.A2.Channel   xMux    "-s$slot  -o0x2c -w16 -prw -b0 -l16 -dCluster -ma2"
	clusterge_register   Insp.A2.Signal    xDig    "-s$slot  -o0x2e -w16 -prw -fd -b0 -l0 -i0"
	clusterge_register   Insp.V.Channel    xMux    "-s$slot  -o0x30 -w16 -prw -b0 -l16 -dCluster -mv"
	clusterge_register   Insp.V.Signal     xDig    "-s$slot  -o0x32 -w16 -prw -fd -b0 -l0 -i0"

	clusterge_register   BufferSetup.Volatile.DAC0   xDig    "-s$slot  -o0x80 -w16 -pwo -fx -b0 -l0 -i255"
	clusterge_register   BufferSetup.Volatile.DAC1   xDig    "-s$slot  -o0x82 -w16 -pwo -fx -b0 -l0 -i255"
	clusterge_register   BufferSetup.Volatile.DAC2   xDig    "-s$slot  -o0x84 -w16 -pwo -fx -b0 -l0 -i255"
	clusterge_register   BufferSetup.Volatile.DAC3   xDig    "-s$slot  -o0x86 -w16 -pwo -fx -b0 -l0 -i255"
	clusterge_register   BufferSetup.Volatile.DAC4   xDig    "-s$slot  -o0x88 -w16 -pwo -fx -b0 -l0 -i255"
	clusterge_register   BufferSetup.Volatile.DAC5   xDig    "-s$slot  -o0x8a -w16 -pwo -fx -b0 -l0 -i255"
	clusterge_register   BufferSetup.Volatile.DAC6   xDig    "-s$slot  -o0x8c -w16 -pwo -fx -b0 -l0 -i255"
	clusterge_register   BufferSetup.Volatile.DAC7   xDig    "-s$slot  -o0x8e -w16 -pwo -fx -b0 -l0 -i255"
	clusterge_register   BufferSetup.Volatile.DAC8   xDig    "-s$slot  -o0x90 -w16 -pwo -fx -b0 -l0 -i255"

	clusterge_register   BufferSetup.PROM.DAC0       xDig    "-s$slot  -o0x60 -w16 -prw -fx -b0 -l0"
	clusterge_register   BufferSetup.PROM.DAC1       xDig    "-s$slot  -o0x62 -w16 -prw -fx -b0 -l0"
	clusterge_register   BufferSetup.PROM.DAC2       xDig    "-s$slot  -o0x64 -w16 -prw -fx -b0 -l0"
	clusterge_register   BufferSetup.PROM.DAC3       xDig    "-s$slot  -o0x66 -w16 -prw -fx -b0 -l0"
	clusterge_register   BufferSetup.PROM.DAC4       xDig    "-s$slot  -o0x68 -w16 -prw -fx -b0 -l0"
	clusterge_register   BufferSetup.PROM.DAC5       xDig    "-s$slot  -o0x6a -w16 -prw -fx -b0 -l0"
	clusterge_register   BufferSetup.PROM.DAC6       xDig    "-s$slot  -o0x6c -w16 -prw -fx -b0 -l0"
	clusterge_register   BufferSetup.PROM.DAC7       xDig    "-s$slot  -o0x6e -w16 -prw -fx -b0 -l0"
	clusterge_register   BufferSetup.PROM.DAC8       xDig    "-s$slot  -o0x70 -w16 -prw -fx -b0 -l0"

	clusterge_register   SAROC1.Test 	xDig "-s$slot  -o0x160 -w32 -pwo -fx -b0 -l0"
	clusterge_register   SAROC1.Offset 	xDig "-s$slot  -o0x164 -w32 -pro -fx -b0 -l0"
	clusterge_register   SAROC2.Test 	xDig "-s$slot  -o0x1e0 -w32 -pwo -fx -b0 -l0"
	clusterge_register   SAROC2.Offset 	xDig "-s$slot  -o0x1e4 -w32 -pro -fx -b0 -l0"

#    common area DACs    

	clusterge_register   CFD.Width         xDAC "-s$slot  -o0x00 -w16 -pwo -qn -us  -d0 -r0:200ns,0xff:0s   -i50ns"
	clusterge_register   TestGenAmpl       xDAC "-s$slot  -o0x02 -w16 -pwo -qk -ueV -d0 -r0:0eV,0xffff:20MeV -i4keV"

#    GIR registers      

	clusterge_register   GIR.EvNum.Count         xDig  "-s$slot  -o0xA0 -w16 -prw -fx -b0 -l0 -i0"
	clusterge_register   GIR.EvNum.Addr          xAdr  "-s$slot  -c0 -o0xA2 -w16 -b0 -l0 -i$registerbase.GIR.EvNum"
	clusterge_register   GIR.EvNum               xDig  "-s$slot  -o0xA0 -w32 -pro -fx -b0 -l0    "
        clusterge_register   GIR.FIFO.Test           xFIT  "-s$slot  -o0xA4 -w32 -prw -fx"
	clusterge_register   GIR.ROTSR               xDig  "-s$slot  -o0xAC -w16 -pro -fx -b0 -l0    "
        clusterge_register   GIR.ValAck.TimeOut      xDAC  "-s$slot -c0 -a0 -z0 -o0xAE -w16 -prw -qn -us -d0 -r0:0s,0xffff:4096000ns -i75000ns"
	clusterge_register   GIR.Qstat0              xDig    "-s$slot  -o0xB0 -w32 -pro -fd -b0 -l16    "
	clusterge_register   GIR.Qstat1              xDig    "-s$slot  -o0xB0 -w32 -pro -fd -b16 -l16    "

	clusterge_register   GIR.ROCR                     xDig    "-s$slot  -o0xA8 -w16 -prw -fx -b0 -l0 -i0"
	clusterge_register   GIR.ROCR.SAROC.Test          xDig    "-s$slot  -o0xA8 -w16 -prw -fx -b0 -l1    "
	clusterge_register   GIR.ROCR.Ext.Test            xDig    "-s$slot  -o0xA8 -w16 -prw -fx -b1 -l1    "
	clusterge_register   GIR.ROCR.ValAck.TimeOutReset xDig    "-s$slot  -o0xA8 -w16 -prw -fx -b2 -l1 -i1"
	clusterge_register   GIR.ROCR.ReadOut.Disable     xDig    "-s$slot  -o0xA8 -w16 -prw -fx -b3 -l1    "
	clusterge_register   GIR.ROCR.PipeLine.Enable     xDig    "-s$slot  -o0xA8 -w16 -prw -fx -b5 -l1    "
	clusterge_register   GIR.ROCR.EvNumData.Disable   xDig    "-s$slot  -o0xA8 -w16 -prw -fx -b6 -l1 -i1"
	clusterge_register   GIR.ROCR.ReadOutMode         xDig    "-s$slot  -o0xA8 -w16 -prw -fx -b7 -l1 -i0"
	clusterge_register   GIR.ROCR.DSPEN               xDig    "-s$slot  -o0xA8 -w16 -prw -fx -b8 -l1 -i0"
	clusterge_register   GIR.ROCR.MODEDMA             xDig    "-s$slot  -o0xA8 -w16 -prw -fx -b9 -l1 -i0"
	clusterge_register   GIR.ROCR.EvNumQbits.Enable   xDig    "-s$slot  -o0xA8 -w16 -prw -fx -b11 -l1 -i1"

	clusterge_register   GIR.ROTSR.Temp.Status           xDig    "-s$slot  -o0xAC -w16 -pro -fx -b0 -l6    "
	clusterge_register   GIR.ROTSR.ValAck.TimeOut.Status xDig    "-s$slot  -o0xAC -w16 -pro -fx -b6 -l1    "
	clusterge_register   GIR.ROTSR.FIFO.Status           xDig    "-s$slot  -o0xAC -w16 -pro -fx -b8 -l4    "
	clusterge_register   GIR.ROTSR.SAROC.Status          xDig    "-s$slot  -o0xAC -w16 -pro -fx -b12 -l1    "
	clusterge_register   GIR.ROTSR.EvFault.Status        xDig    "-s$slot  -o0xAC -w16 -pro -fx -b13 -l3    "

#    GIR DSP Registers

        clusterge_register GIR.DSP.Control          xDig "-s$slot -a0x400000 -o0xbff40 -w32 -prw -fx -b0 -l0"
        clusterge_register GIR.DSP.StartDSP         xDig "-s$slot -a0x400000 -o0xbff40 -w32 -prw -fd -b16 -l1"
        clusterge_register GIR.DSP.StopDSP          xDig "-s$slot -a0x400000 -o0xbff40 -w32 -prw -fd -b18 -l1"
        clusterge_register GIR.DSP.ResetDSP         xDig "-s$slot -a0x400000 -o0xbff40 -w32 -prw -fd -b20 -l1"
        clusterge_register GIR.DSP.Mux              xDig "-s$slot -a0x400000 -o0xbff40 -w32 -prw -fd -b21 -l1"
        clusterge_register GIR.DSP.Pz               xDig "-s$slot -a0x400000 -o0xbff40 -w32 -prw -fd -b22 -l1" 
       	clusterge_register GIR.DSP.ResetMemExt          xDig "-s$slot -a0x400000 -o0xbff40 -w32 -prw -fd -b23 -l1"
        clusterge_register GIR.DSP.Control.FilterMode   xDig "-s$slot -a0x400000 -o0xbff40 -w32 -prw -fd -b17 -l1"
        clusterge_register GIR.DSP.Control.DSPtotcount  xDig "-s$slot -a0x400000 -o0xbff40 -w32 -prw -fd -b19 -l1"
        clusterge_register GIR.DSP.Mirror               xDig "-s$slot -a0x400000 -o0xbff44 -w32 -prw -fx -b0 -l0"
        clusterge_register GIR.DSP.Item                 xDig "-s$slot -a0x400000 -o0x98004 -w32 -prw -fx -b0 -l0"
	clusterge_register GIR.DSP.Spectre          xDig    "-s$slot  -a0x400000 -o0xbff40 -w32 -prw -fd -b24 -l1"
	clusterge_register GIR.DSP.Scaler           xDig    "-s$slot  -a0x400000 -o0xbff40 -w32 -prw -fd -b25 -l1"

        clusterge_register GIR.DSP.Program          xGIR "-s$slot -a0x400010"
        clusterge_register GIR.DSP.ExtMem           bVXI "-s$slot -a0x200000 -o0x0 -w32 -pro -fx -l64"
        clusterge_register GIR.DSP.SpectrumData     bVXI "-s$slot -a0x200000 -o0x0 -w32 -pro -fx -l64"
        clusterge_register GIR.DSP.IntMem           bVXI "-s$slot -a0x400000 -o0x0 -w32 -pro -fx -l64"

 }

proc ClusterGeDetectorRegisters {slot c} {

        clusterge_register   CFD.Thresh   xRng "-s$slot -c$c -a0x1000 -z0x100 -o0x00 -G0x10 -b1 -qk -ueV -d0 -r0:2.6MeV,0xff:0eV -R0:825keV,0xff:0eV -I1 -i50keV"
        clusterge_register   CFD.Delay    xDAC "-s$slot -c$c -a0x1000 -z0x100 -o0x02 -w16 -pwo -qn -us -d0 -r0:0s,0xff:200ns -i100ns"
        clusterge_register   PoleZero     xDig "-s$slot -c$c -a0x1000 -z0x100 -o0x04 -w16 -pwo -fd -l8 -i200"
        clusterge_register   FTSample     xDAC "-s$slot -c$c -a0x1000 -z0x100 -o0x06 -w16 -pwo -qn -us  -d0 -r0:0s,0xff:3.2us -i515ns"
        clusterge_register   ValSample    xDAC "-s$slot -c$c -a0x1000 -z0x100 -o0x08 -w16 -pwo -qn -us  -d0 -r0:0s,0xff:16us  -i2.5us"
        clusterge_register   Watchdog     xDAC "-s$slot -c$c -a0x1000 -z0x100 -o0x0c -w16 -pwo -qn -us  -d0 -r0:0s,0xff:160us -i120us"
        clusterge_register   PileUpThresh xDAC "-s$slot -c$c -a0x1000 -z0x100 -o0x0a -w16 -pwo -qu -uV  -d0 -r0:-50mV,0xff:50mV -i-8mV"
        clusterge_register   BipolarCO    xDig "-s$slot -c$c -a0x1000 -z0x100 -o0x0e -w16 -pwo -fd -l8 -i0x16"

        clusterge_register   CCR          xDig "-s$slot -c$c -a0x1000 -z0x100 -o0x10 -w16 -prw -fx -b0 -l0"
        clusterge_register   Enable       xDig "-s$slot -c$c -a0x1000 -z0x100 -o0x10 -w16 -prw -fd -b0 -l1 -i0"
        clusterge_register   TestMode     xDig "-s$slot -c$c -a0x1000 -z0x100 -o0x10 -w16 -prw -fd -b1 -l1 -i0"
}

proc ClusterGeSAROCRegisters {slot c a} {
   global registerbase

    clusterge_register 4MeV.Addr      xAdr    "-s$slot -c$c -a$a -z 24 -o0x00 -w32 -i$registerbase.4MeV"
    clusterge_register 4MeV.TestData  xDig    "-s$slot -c$c -a$a -z 24 -o0x00 -w32 -g0 -prw -fx -b16 -l0"
    clusterge_register 4MeV.TestRd    xDig    "-s$slot -c$c -a$a -z 24 -o0x00 -w32 -g0 -pro -fx -b16 -l0"
    clusterge_register 4MeV.Thresh    xDig    "-s$slot -c$c -a$a -z 24 -o0x04 -w32 -g1 -prw -fd -b0  -l16 -i0x0"
    clusterge_register 4MeV.ShiftCtrl xDig    "-s$slot -c$c -a$a -z 24 -o0x04 -w32 -g1 -prw -fd -b16 -l2 -i1"
    clusterge_register 4MeV.ROEnable  xDig    "-s$slot -c$c -a$a -z 24 -o0x04 -w32 -g1 -prw -fd -b18 -l1 -i1"
    clusterge_register 4MeV.SSEnable  xDig    "-s$slot -c$c -a$a -z 24 -o0x04 -w32 -g1 -prw -fd -b19 -l1 -i1"
    clusterge_register 4MeV.OSDisable xDig    "-s$slot -c$c -a$a -z 24 -o0x04 -w32 -g1 -prw -fd -b20 -l1 -i0"

    clusterge_register 20MeV.Addr      xAdr    "-s$slot -c$c -a$a -z 24 -o0x08 -w32 -i$registerbase.20MeV"
    clusterge_register 20MeV.TestData  xDig    "-s$slot -c$c -a$a -z 24 -o0x08 -w32 -g0 -prw -fx -b16 -l0"
    clusterge_register 20MeV.TestRd    xDig    "-s$slot -c$c -a$a -z 24 -o0x08 -w32 -g0 -pro -fx -b16 -l0"
    clusterge_register 20MeV.Thresh    xDig    "-s$slot -c$c -a$a -z 24 -o0x0c -w32 -g1 -prw -fd -b0  -l16 -i0x0"
    clusterge_register 20MeV.ShiftCtrl xDig    "-s$slot -c$c -a$a -z 24 -o0x0c -w32 -g1 -prw -fd -b16 -l2 -i1"
    clusterge_register 20MeV.ROEnable  xDig    "-s$slot -c$c -a$a -z 24 -o0x0c -w32 -g1 -prw -fd -b18 -l1 -i1"
    clusterge_register 20MeV.SSEnable  xDig    "-s$slot -c$c -a$a -z 24 -o0x0c -w32 -g1 -prw -fd -b19 -l1 -i1"
    clusterge_register 20MeV.OSDisable xDig    "-s$slot -c$c -a$a -z 24 -o0x0c -w32 -g1 -prw -fd -b20 -l1 -i0"

    clusterge_register FT.Addr      xAdr    "-s$slot -c$c -a$a -z 24 -o0x10 -w32 -i$registerbase.FT"
    clusterge_register FT.TestData  xDig    "-s$slot -c$c -a$a -z 24 -o0x10 -w32 -g0 -prw -fx -b16 -l0"
    clusterge_register FT.TestRd    xDig    "-s$slot -c$c -a$a -z 24 -o0x10 -w32 -g0 -pro -fx -b16 -l0"
    clusterge_register FT.Thresh    xDig    "-s$slot -c$c -a$a -z 24 -o0x14 -w32 -g1 -prw -fd -b0  -l16 -i0x0"
    clusterge_register FT.ShiftCtrl xDig    "-s$slot -c$c -a$a -z 24 -o0x14 -w32 -g1 -prw -fd -b16 -l2 -i1"
    clusterge_register FT.ROEnable  xDig    "-s$slot -c$c -a$a -z 24 -o0x14 -w32 -g1 -prw -fd -b18 -l1 -i1"
    clusterge_register FT.SSEnable  xDig    "-s$slot -c$c -a$a -z 24 -o0x14 -w32 -g1 -prw -fd -b19 -l1 -i1"
    clusterge_register FT.OSDisable xDig    "-s$slot -c$c -a$a -z 24 -o0x14 -w32 -g1 -prw -fd -b20 -l1 -i0"
}



proc ClusterGe_DAQ-reset   {args}   {
   set module [first $args]
#    flush FIFOs
   write-vxi-register $module.FIFO.Reset 0
   write-vxi-register $module.FIFO.Reset 1
   write-vxi-register $module.FIFO.Reset 0
#    reset and enable GIR
   write-vxi-register $module.GIR.FIFO.Test 0
   read-vxi-register $module.GIR.FIFO.Test
}


proc ClusterGe_DAQ-halt   {args}   {
   set module [first $args]
#    no action
}


proc ClusterGe_DAQ-go   {args}   {
   set module [first $args]
#    zero event numbers
   write-vxi-register $module.GIR.EvNum.Count 0
}

