#dqc.tcl -- detector quality tester tcl (loops over set of detectors)

set debug 0

set framename DQC
set frametitle "Detector Quality Checker"
set started 0

set specserver 0
set specservernames {/hist /disc}
set specservername ""
set specpath ""
set specdir ""

set goodlog ""
set errlog ""

set selectopt 0
set selectoptnames {
   *.4MeV.q0+2
   *.4MeV.q0
   Clover*.4MeV.q0
   Ge*.4MeV.q0
   Clover*.GeA.4MeV.q0
   Clover*.GeB.4MeV.q0
   Clover*.GeC.4MeV.q0
   Clover*.GeD.4MeV.q0
   custom
   }
set patternmatch ""

#limit to Co60 for now ...

set source 0
set sourcenames {Co60}

set calmode 0
set calmodes { "Already Calibrated" "Not Calibrated" }

set cutoff 100

set confirmcontinue ""

set details 0
set selection {}
set filelist {}
set dirlist {}

# Co60 parameters
set co(0) 1173.0; set co(1) 1332.0

set peak(C) 0.0; set peak(deltaC) 0.0
set peak(E) 0.0; set peak(deltaE) 0.0
set peak(BA) 0.0; set peak(PA) 0.0; set peak(W) 0.0

set fwhm {}
set fwtm {}
set ptot {}

set coef(A) 0.0; set coef(B) 0.0; set coef(C) 0.0; set coef(D) 0.0

set seg(phase1)  0
set seg(A)       0
set seg(B)       1
set seg(C)       2
set seg(D)       3

set segletters {A B C D}

set dqcx 180
set dqcw 37

set dqcformat \
  "(:frame DQC :label 'Detector Quality Checker'
         :show-footer T :shrink-wrap T  :resize F
         :icon (:icon :image eg-array :transparent T :label 'DQC')
    (:panel DQC-PANEL1 :x 0 :y 0 :w 500 :h 300 :row-gap 15 :layout V
       (:choice-stack DQC-SPECSERVER :label 'Spectrum server:' :valuex $dqcx
              :valuey 10
              :value 0)
       (:text DQC-SPECDIR :label 'Spectrum directory:' :w $dqcw :valuex $dqcx)
       (:choice-stack DQC-SELECTOPT :label 'Selection pattern:' :valuex $dqcx
              :value 0)
       (:text DQC-SELECTSTRING :label 'Custom pattern match:' :w $dqcw :valuex $dqcx
              :read-only T)
       :row-gap 30
       (:choice-stack DQC-SOURCE :label 'Calibration source:' :valuex $dqcx
              :value 0)
       :row-gap 15
       (:number DQC-CUTOFF :label 'Low energy cutoff (keV):' :w 4 :valuex $dqcx
               :min 0 :max 1024 :value $cutoff)
       :row-gap 15
       (:choice-stack DQC-MODE :label 'Calibration mode:' :valuex $dqcx
              :value 0)
    )
    (:panel DQC-PANEL4 :x 500 :y 10 :w 220 :h 290
       (:message :label '       matched spectra' :x 10 :y 0 :bold T)
       (:list DQC-SPECLIST :x 0 :y 15 :rows 15 :list-width 200 :strings('')
              :read-only T)
    )
    (:panel DQC-PANEL5 :x 0 :y 250 :w 500 :h 50
       (:button DQC-HELP       :label 'Help'      :x  50  :y 10)
       (:button DQC-RUN        :label 'Run'       :x 200  :y 10)
       (:button DQC-REFRESH    :label 'Refresh' :x 350  :y 10)
    )
    (:panel :x 0 :y 300 :w 720 :h 20 :layout H
       (:message DQC-PANEL6 :bold T :deaf T :valuex 460 :valuey 6
             :label 'E R R O R    M E S S A G E S')
    )
    (:galley DQC-GALLEY :x 0 :y 320 :read-only F :rows 20 :cols 100)
    :show T)"

#------------------------------------------------------------------------------
proc first p {lindex $p 0}
proc second p {lindex $p 1}
proc third p {lindex $p 2}
proc fourth p {lindex $p 3}

proc insert-log s {
   send sl insert [transparent $s]
   }

proc quote-list l {
   set ll ""
   foreach f $l {append ll " " "'$f'"}
   return $ll
   }

proc quote-strings l {
   set ll ""
   foreach f $l {append ll [format "'%s' " $f]}
   return $ll
   }

proc init-dqc-frame {} {
   global debug
   global errorCode
   global started framename
   global env
   global dqcformat
   global specserver specservernames specservername
   global patternmatch selectopt selectoptnames
   global source sourcenames sourcename
   global calmode calmodes cutoff

   if {$debug} then {puts stderr "init-dqc-frame"}
   set started 1

# display window
   xv $dqcformat
   xv "(DQC :busy T)"
   xv "(DQC-SPECSERVER :strings ([quote-list $specservernames]) :value 0)"
   xv "(DQC-SELECTOPT :strings ([quote-list $selectoptnames]) :value 0)"
   xv "(DQC-SOURCE :strings ([quote-list $sourcenames]) :value 0)"
   xv "(DQC-MODE :strings ([quote-list $calmodes]) :value 0)"

# tcl-shell-0.999 has bug fix for setting of errorCode by exec:
# Initialise errorCode variable in case it is tested and before an error occurs
   set z [catch {error "set errorCode"} m]

# initialise widget values
   set specservername [lindex $specservernames $specserver]

   if {$selectopt < 7} then {
      set patternmatch [lindex $selectoptnames $selectopt]
      }

   set sourcename [lindex $sourcenames $source]
	
   set calmode 0

   do-repaint
   xv "(DQC :busy F)"
   }

proc bring-to-front f {
   global debug
   if {$debug} then {puts stderr "bring-to-front: $f"}
   xv "($f :closed F)"
   }

proc set-footer m {
   global framename
   xv "($framename :left-footer '[quote $m]')"
   }

proc clear-footer {} {
   global framename
   xv "($framename :left-footer ' ')"
   }

#------------------------------------------------------------------------------
proc click-specserver v {
   global debug
   global framename specserver specservernames specservername
   if {$debug} {puts stderr "click-specserver: '$v'"}
   xv "($framename :busy T)"
   clear-footer
   set specserver $v
   set specservername [lindex $specservernames $specserver]
   xv "($framename-SPECSERVER :value $specserver)"
   xv "($framename :busy F)"
   }

proc click-specdir s {
   global debug
   global framename specdir
   if {$debug} {puts stderr "click-specdir: '$s'"}
   xv "($framename :busy T)"
   clear-footer
   set specdir $s
   read-selected-spectra
   paint-selected-spectra
   xv "($framename-SPECDIR :value '$specdir')"
   xv "($framename :busy F)"
   }

proc click-source v {
   global framename source sourcenames sourcename
   xv "($framename :busy T)"
   clear-footer
   set source $v
   set sourcename [lindex $sourcenames $source]
   xv "($framename-SOURCE :value $source)"
   xv "($framename :busy F)"
   }

proc click-mode v {
   global framename calmode
   xv "($framename :busy T)"
   clear-footer
   set calmode $v
   xv "($framename-MODE :value $calmode)"
   xv "($framename :busy F)"
   }

proc click-selectopt v {
   global debug
   global framename selectopt selectoptnames patternmatch
   if {$debug} {puts stderr "click-selectopt: '$v'"}
   xv "($framename :busy T)"
   clear-footer
   set selectopt $v
   set patternmatch [lindex $selectoptnames $v]
   if {$debug} {puts stderr "...patternmatch: $patternmatch"}
      xv "($framename-SELECTOPT :value $selectopt)"
   if {$patternmatch != "custom"} {
      xv "($framename-SELECTSTRING :value '$patternmatch' :read-only T)"
      } \
   else {
      set patternmatch ''
      xv "($framename-SELECTSTRING :value '' :read-only F)"
      }
   read-selected-spectra
   paint-selected-spectra
   xv "($framename :busy F)"
   }

proc click-selectstring s {
   global debug
   global framename patternmatch
   if {$debug} {puts stderr "click-selectstring: '$s'"}
   xv "($framename :busy T)"
   clear-footer
   set patternmatch $s
   xv "($framename-SELECTSTRING :value '$patternmatch')"
   read-selected-spectra
   paint-selected-spectra
   xv "($framename :busy F)"
   }

#------------------------------------------------------------------------------
proc sort-filelist {} {
   global filelist
   set ll {}
   foreach f $filelist {lappend ll [second $f]/[first $f]}
   set ll [lsort $ll]
   set filelist {}
   foreach f $ll {
      set s [split $f "/"]
      lappend filelist [list [second $s] [first $s]]
      }
   }

proc spec-type t {
   if {$t==1} then {return s}
   if {$t==2} then {return d}
   return ?
   }

proc make-path {} {
   global specpath
   global specservername specdir
   if {$specservername=="/hist"} then {
         set specpath "/hist"
      } \
   else {
         set specpath $specservername/$specdir
      }
   }

proc make-details s {
   global debug
   global specpath
   make-path
   if {$debug} then {puts stderr "make-details: $specpath/$s"}
   set z [catch {eg "(spectrum :name '$specpath/$s' ?spec)"} m]
   if {$z!=0} then {
      display-eg-error-report self $m "Attempting to read spectrum details" \
                                      $specpath/$s
      clear-footer
      error $m
      }
   set dim [first $m]
   set bases [second $m]
   set ranges [third $m]
   set ll ""
   if {$dim>=1} then {append ll "[first $bases]:[first $ranges]"}
   if {$dim>=2} then {append ll "x[second $bases]:[second $ranges]"}
   if {$dim>2} then {append ll "x..."}
   return $ll
   }

proc read-directory {} {
   global debug
   global specpath filelist
   make-path
   if {$debug} then {puts stderr "read-directory: $specpath"}
   set z [catch {eg "(spectrum :path '$specpath' ?names)"} m]
   if {$z!=0} then {
      display-eg-error-report self $m "Attempting to read directory" \
                                      $specpath
      clear-footer
      error $m
      }
   if {$debug} then {puts stderr "read-directory: $m"}
   set filelist $m
   sort-filelist
   make-dirlist
   }

proc select-matching-files {} {
   global filelist patternmatch selection nselected
   if {$patternmatch==""} then {set selection {}; return}
   set ll {}; set i 0; set nselected 0
   foreach f $filelist {
      set n [second $f]
      if {$n!="." && $n!=".."} then \
         {
         if {[match-name $patternmatch $n]} then {lappend ll $i; incr nselected}
         incr i
         }
      }
   set selection $ll
   }

proc read-selected-spectra {} {
   global debug
   global selectlist selection dirlist
   if {$debug} {puts stderr "read-selected-spectra"}
   read-directory
   select-matching-files
   set selectlist {}
   foreach i $selection {
      lappend selectlist [second [lindex $dirlist $i]]
      }
   }

proc paint-selected-spectra {} {
   global debug
   global selectlist nselected
   if {$debug} {puts stderr "paint-selected-spectra"}
   xv "(DQC-SPECLIST :show F :strings ([quote-list $selectlist]) :scroll-from 0 :show T)"
   xv "(DQC :left-footer '$nselected spectra matching selection pattern')"
   }

proc make-selected-spectra {} {
   global dirlist selection
   set ll {}
   foreach i $selection {
      set n [second [lindex $dirlist $i]]
      lappend ll $n
      }
   return $ll
   }

proc make-dir-line {t s} {
   global debug
   global details
   if {$debug>3} then {puts stderr "make-dir-line: $t $s"}
   if {$s=="." || $s==".."} then {return ""}
   if {$t==2} then {return [format "%s   %20s" [spec-type $t] $s]}
   if {$t==1} then {
      if {$details} then {return [format "%s   %20s   %s" \
            [spec-type $t] $s [make-details $s]] } \
      else {return [format "%s   %20s" [spec-type $t] $s]}
      }
   return ""
   }

proc make-dirlist {} {
   global debug
   global filelist dirlist global dircount details
   set dirlist {}; set dircount 0
   if {$debug>2} then {puts stderr "make-dirlist: filelist=$filelist"}
   foreach p $filelist {
      set t [first $p]
      set s [second $p]
      set l [make-dir-line $t $s]
      if {$l!=""} then {
         lappend dirlist $l
         incr dircount
         }
      }
   if {$debug>2} then {puts stderr "...dirlist=$dirlist"}
   }

proc abs x {if {$x>=0} then {return $x} else {return [expr 0-$x]}}

proc list-max s {
   set m [first $s]
   foreach i $s {if {$i>$m} then {set m $i}}
   return $m
   }

proc list-min-index s {
   set m [first $s]; set i 0; set j 0
   foreach n $s {if {$n<$m} then {set m $n; set j $i}; incr i}
   return $j
   }
 
proc list-nearest {s v d} {
   set ds {}
   foreach i $s {lappend ds [abs [expr $v-$i]]}
   set i [list-min-index $ds]
   set dd [lindex $ds $i]
   if {$dd>$d} then {error "closest is $i with delta $d"}
   return [lindex $s $i]
   }

proc split-at-equal s {split $s "="}

proc trim s {string trim $s}

proc map {p ss} {
   set ll {}
   foreach s $ss {lappend ll [$p $s]}
   return $ll
   }

proc scan-float s {
   set f 0.0
   scan $s "%f" f
   return $f
   }

proc parse-peak-integrate pp {
   global peak
   foreach p $pp {
      if {[llength $p]<2} then {continue}
      set value 0.0; set value2 0.0
      set label [trim [first $p]]
      set value [scan-float [second $p]]
      set vals [split [second $p] "/"]
      if {[llength $vals]>=2} then \
         {set value2 [scan-float [string trimleft [second $vals] "-"]]}
      if {$label=="Centroid"} then \
         {set peak(C) $value; set peak(deltaC) $value2}
      if {$label=="Energy"} then \
         {set peak(E) $value; set peak(deltaE) $value2}
      if {$label=="Background area"} then {set peak(BA) $value}
      if {$label=="Peak area"} then {set peak(PA) $value}
      if {$label=="Peak width"} then {set peak(W) $value}
      }
   }

proc parse-detcheck pp {
   global fwhm fwtm ptot
   set fwhm {}
   set fwtm {}
   foreach p $pp {
      if {[llength $p]<2} then {continue}
      set value 0.0; set value2 0.0
      set label [trim [first $p]]
      set value [scan-float [second $p]]
      set value2 [scan-float [second [second $p]]]
      if {$label=="FWHM"} then \
         {lappend fwhm $value; lappend fwhm $value2}
      if {$label=="FWTM"} then \
         {lappend fwtm $value; lappend fwtm $value2}
      if {$label=="Peak/Total"} then \
         {set ptot $value}
      }
   }

proc parse-calibrate pp {
   global coef
   foreach p $pp {
      if {[llength $p]<2} then {continue}
      set value 0.0; set value2 0.0
      set label [trim [first $p]]
      set value [scan-float [second $p]]
      set vals [split [second $p] "/"]
      if {[llength $vals]>=2} then \
         {set value2 [scan-float [string trimleft [second $vals] "-"]]}
      if {$label=="A"} then \
         {set coef(A) $value; set coef(deltaA) $value2}
      if {$label=="B"} then \
         {set coef(B) $value; set coef(deltaB) $value2}
      if {$label=="C"} then \
         {set coef(C) $value; set coef(deltaC) $value2}
      if {$label=="D"} then \
         {set coef(D) $value; set coef(deltaD) $value2}
      }
   }

proc make-calibrate-input {cs deltacs es} {
   global debug
   set lines ""
   for {set i 0} {$i<[llength $cs]} {incr i} {
      append lines "[lindex $cs $i] [lindex $es $i] [lindex $deltacs $i] 0.1 \n"
      }
   if {$debug} then {puts stdout "$lines"}
   return $lines
   }

proc update-calibrations {} {
#   xv "([selection frame]-GALLERY :refresh-calibrations T :refresh-tags T)"
   }

proc put-log text {
   global framename goodlog
   xv "($framename-GALLEY :insert '[quote $text]' :insert '\n')" 
   lappend goodlog $text
   }

proc put-errorlog text {
   global framename errlog
   lappend errlog $text
   }

#==============================================================================
   
proc confirm m {
   global debug
   global framename
   xv "($framename (:notice $framename-NOTICE :string '[quote $m]' \
:no 'Cancel' :yes 'Proceed' :show T))"
   if {$debug} then {puts stderr "confirm: $m"}
   }

proc click-run {} {
   global debug
   global confirmcontinue

   confirm "WARNING!!  Take care if running this from your experiment control session since it can take several minutes and will freeze all other EG-session windows!"
   set confirmcontinue "do-detcheck-all"
   }

proc do-detcheck-all {yesorno} {
   global debug errlog
   global framename specpath
   global sourcename
   global co peak coef fwhm fwtm ptot
   global nselected
   global calmode cutoff
   
   if {$yesorno!=1} then {xv "($framename :busy F)"; return}

#------------------------------------------------------------------------------
# this function outputs the modified gain coefficients for each
# spectrum in the list by performing an auto-calibration on it and
# theory: Egam_calib = coef(A) + coef(B)*chan + coef(C)*chan*chan
#         Egam_observed  = Egam_reaction * (1 + v/c*cos(theta))
#      => shifted coefficient = calib_coeff / (1+v/c*cos(theta))
#------------------------------------------------------------------------------

   xv "($framename-GALLEY :erase T)"
   set goodlog ""
   set errlog ""

   xv "($framename :busy T)"
   clear-footer

#------------------------------------------------------------------------------

# refresh spectrum list and update selected spectrum list...
   read-selected-spectra
   paint-selected-spectra
   set-footer "Running detector quality check on $nselected spectra..."

#------------------------------------------------------------------------------

# compile list of spectrum names matching specified pattern option...
   set ss [make-selected-spectra]
   if {$debug} {puts stderr "ss = $ss"}

#------------------------------------------------------------------------------

# loop over spectra, save gain coefficients...

   set seg_list(phase1) {}
   set seg_list(A) {}
   set seg_list(B) {}
   set seg_list(C) {}
   set seg_list(D) {}
   set seg_list(Amod) {}
   set seg_list(Bmod) {}
   set seg_list(Cmod) {}
   set seg_list(Dmod) {}

   put-log "Detector Quality Check Results:"
   put-log ""
   put-log "    Spectrum name   Centroids(chans)   FWHM (keV)   FWTM (keV)   FWTM/FWHM    Peak/Total"
   put-log "----------------------------------------------------------------------------------------"

   foreach s $ss {
      make-path

      set-footer "running detector quality check on $specpath/$s..."

      set goto_next_spectrum 0

#------------------------------------------------------------------------------

# calibrate spectrum according to source option...
      set hw 20
      
      set-footer "finding Cobalt peaks for spectrum $s..."
      set z [catch {exec peaklist  \
            -spectrum $specpath/$s -limit 1 4095 -peaks 2} m]
      if {$z!=0} then {
         put-errorlog "$s  FAULTY ... Peakfind error: $m"
         continue
         }
      if {[llength $m]!=2} then {
         put-errorlog "$s  FAULTY ... can't find the two Cobalt peaks"
         continue
         }
      set tags $m

      set centroids {}
      set deltacentroids {}
      set widths {}
      set taglist {}

      foreach i {0 1} {
         set c [lindex $tags $i]
         set c1 [expr $c-$hw]
         set c2 [expr $c+$hw]
         set-footer "taking $co($i)keV peak centroid at channel $c..."
         set z [catch {exec integrate -spectrum $specpath/$s -limit $c1 $c2} m]
         if {$z!=0} then \
           {
           put-errorlog "$s  FAULTY ... Integration error: $m"
           set goto_next_spectrum 1
           break
           }
         parse-peak-integrate [map split-at-equal [split $m ",\n"]]
         lappend centroids $peak(C)
         lappend deltacentroids $peak(deltaC)
         lappend widths $peak(W)
         lappend taglist $c1
         lappend taglist $c2
         }

      if {$goto_next_spectrum} then \
         continue

      if {$debug>1} then {puts stderr "centroids are $centroids"
         puts stderr "deltacentroids are $deltacentroids"
         puts stderr "widths are $widths"
         puts stderr "tags are $taglist"
         }
   
      if {$calmode} then {
         set-footer "computing calibration..."

         set energies [list $co(0) $co(1)]
         set z [catch {exec calibrate -spectrum $specpath/$s \
            -opt linear  -units keV \
            << [make-calibrate-input $centroids $deltacentroids $energies] } m]

         if {$z!=0} then {
            put-errorlog "$s  FAULTY ... Calibration error: $m"
            continue
            }
         insert-log $m
  
         update-calibrations

         parse-calibrate [map split-at-equal [split $m ",\n"]]
         if {$debug>1} then {
            puts stderr "coef: A=$coef(A) B=$coef(B) C=$coef(C) D=$coef(D)"
            }
         }

# Now enter detcheck program with calibrated spectra ...

      set-footer "Calculating detector params for spectrum $s..."
      set z [catch {exec detcheck -spectrum $specpath/$s -cutoff $cutoff \
          -limit [first $taglist] [second $taglist] [third $taglist] [fourth $taglist]} m]
      if {$z!=0} then {
         put-errorlog "$s  FAULTY ... $m"
         continue
         }

      parse-detcheck [map split-at-equal [split $m ",\n"]]
      if {$debug>1} then {
         puts stderr "fwhm: $fwhm fwtm: $fwtm ptot: $ptot%"
         }

      set fwparm1 [expr [first $fwtm]/[first $fwhm]]
      set fwparm2 [expr [second $fwtm]/[second $fwhm]]

      put-log [format " %16s   %7.2f %7.2f   %5.2f %5.2f   %5.2f %5.2f  %5.2f %5.2f    %5.2f" $s [first $centroids] [second $centroids] [first $fwhm] [second $fwhm] [first $fwtm] [second $fwtm] $fwparm1 $fwparm2 $ptot]
      
     }

   put-log ""
   put-log "Problems found ..."
   put-log "----------------------------------------------------------------------------------------"
   foreach f $errlog {put-log $f}
   put-log "----------------------------------------------------------------------------------------"

   clear-footer
   xv "($framename :busy F)"

   global confirmcontinue

   confirm "Save performance data to disc file?"
   set confirmcontinue "do-detcheck-save"
   }

proc do-detcheck-save {yesorno} {
    global env patternmatch goodlog
    if {$yesorno!=1} then {xv "($framename :busy F)"; return}
    set ds [exec date]
    set date [third $ds][second $ds][sixth $ds]
    set filename $date.$patternmatch
    set path /home/$env(USER)/geperf
    catch {exec mkdir $path}
    set-footer "Saving to file $path/$filename ..."

    set f [open $path/$filename w]
    foreach l $goodlog {
    puts $f $l
    }
    close $f
}


#------------------------------------------------------------------------------

proc null-action {} {
   }

proc click-refresh {} {
   global debug
   global framename
   if {$debug} {puts stderr "click-refresh: $framename"}
   xv "($framename :busy T)"
   clear-footer
   do-repaint
   xv "($framename :busy F)"
   }

proc do-repaint {} {
   global framename
   global specserver specdir selectopt patternmatch
   global source
   xv "($framename-SPECSERVER :value $specserver)
       ($framename-SPECDIR :value '$specdir')
       ($framename-SELECTOPT :value $selectopt)
       ($framename-SELECTSTRING :value '$patternmatch')
       ($framename-SOURCE :value $source)"
   read-selected-spectra
   paint-selected-spectra
   }

proc click-help f {
   send mb goto [transparent "(SX)Gainmatch Setup window"]
   }

proc click-quit {} {
    global started framename
    set started 0
    xv "($framename :delete)"
}

proc click arg {
   global debug confirmcontinue
   global cutoff
   if {$debug} then {puts stderr "click with argument \{$arg\}" }
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$f=="ER"} then {remove-eg-error-report self; return}
   if {$c==":quit"} then {click-quit; return}
   case $w in {
      DQC-SPECSERVER   {click-specserver $v}
      DQC-SPECDIR      {click-specdir $v}
      DQC-SOURCE       {click-source $v}
      DQC-CUTOFF       {set cutoff $v}
      DQC-MODE         {click-mode $v}
      DQC-SELECTOPT    {click-selectopt $v}
      DQC-SELECTSTRING {click-selectstring $v}
      DQC-SPECLIST     {null-action}
      DQC-REFRESH      {click-refresh}
      DQC-RUN          {click-run}
      DQC-NOTICE       {$confirmcontinue $v}
      DQC-HELP         {click-help $f}
      default         {puts stdout $arg}
      }
   }

proc self op {
   global framename frametitle
   case $op in {
      frame-name {return $framename}
      frame-title {return $frametitle}
      }
   }

proc start args {
   global started framename
   if {$started} then {bring-to-front $framename} \
   else {init-dqc-frame}
   }

register-interest dqc

