#    hist task code overlay for detector type "cluster"


set lowchannel   0
set highchannel 15
set maxchannel  15
set qualifier    1

set genames     {20MeV 4MeV FT}
set bgonames    {Energy TAC}
set sumnames    {Energy TAC Pattern}
set girnames    {GIR.EvNum}

set qnames    {q0 q1 q2 q3}
set qualnames {0 1 2 3}

set ge  0
set bgo 0
set sum 0
set gir 0

set gesegnames {A B C D E F G}
set geseg 0x7f

set bgosegnames {A B C D E F G H I J K L M N O P Q R}
set bgoseg 0


proc make-overlaypanel {} {
   global gesegnames geseg bgosegnames bgoseg
   global genames bgonames sumnames girnames qnames

    :layout V; :row-gap 5; :col-gap 0

      :message DET-M1  :label "Cluster detector overlay applied"

      :number DET-LOW  :label "Low channel:"   :w 5 :valuex 15c
      :number DET-HIGH :label "High channel:"  :w 5 :valuex 15c

      :choice DET-GESEG   :label "GE Segment:"       :valuex 15c :exclusive F :strings $gesegnames :value $geseg
      :choice DET-BGOSEG  :label "BGO Segment:"      :valuex 15c :exclusive T :strings [list All None Select] :value 1
      :layout H
      :hskip 10
      :choice DET-GIR  :label "GIR" :exclusive F :strings [list EvNum] :value 0
      :next-row T
      :choice DET-QUAL :label "Qualifier"  :valuex 15c :exclusive F :strings $qnames :value 1
      :next-row T
      :choice DET-GE   :label "ADC - Ge:"  :valuex 15c :exclusive F :strings $genames :value 0
      :next-row T
      :choice DET-BGO  :label "ADC - BGO:" :valuex 15c :exclusive F :strings $bgonames :value 0
      :next-row T
      :choice DET-SUM  :label "ADC - Sum BGO:"   :valuex 15c :exclusive F :strings $sumnames :value 0
}

proc make-SELframe {} {
  global bgosegnames bgoseg

   :frame SEL :label "Cluster BGO Segment"  :show-footer F :resize F :transient T
   :panel SEL-PANEL :layout T
   :choice DET-BGOSEL :exclusive F  :strings $bgosegnames  :value $bgoseg
   :show T
}


#    this proc handles events from widgets created in the proc make-overlaypanel
#    The widgets DET-LOW; DET-HIGH and DET-QUAL are handled in the main click procedure
#    and are not required here

proc click-overlay {w v} {

   switch -glob $w {
      DET-GE           {click-adc $v ge}
      DET-BGO          {click-adc $v bgo}
      DET-SUM          {click-adc $v sum}
      DET-GIR          {click-adc $v gir}
      DET-GESEG        {click-segment $v}
      DET-BGOSEG       {click-bgo-segment $v}
      DET-BGOSEL       {click-bgo-select $v}
      default      {puts stdout $arg}
   }
}

proc repaint_overlay {} {
   global spectrumlist
   global ge bgo sum gir seg qualifier geseg bgoseg

   if {[llength $spectrumlist] > 0} then {
       :modify DET-GE  :value 0
       :modify DET-BGO :value 0
       :modify DET-SUM :value 0
       :modify DET-GIR :value 0
       :modify DET-QUAL :value 0
       :modify DET-GESEG :value 0
       :modify DET-BGOSEG :value 0
   } else {
       :modify DET-GE  :value $ge
       :modify DET-BGO :value $bgo
       :modify DET-SUM :value $sum
       :modify DET-GIR :value $gir
       :modify DET-QUAL :value $qualifier
       :modify DET-GESEG :value $geseg
       :modify DET-BGOSEG :value $bgoseg
   }
}


proc get-spectrum-list {} {
        global spectrumlist
        global genames bgonames sumnames girnames ge bgo sum gir
        global qualifier qualnames
        global geseg gesegnames bgoseg bgosegnames
        global lowchannel highchannel

        if {$spectrumlist != {}} then {return $spectrumlist};  # return names selected from spectrum list

        set qlist   [list-take $qualnames [bits-in $qualifier]]
        set geseglist [list-take $gesegnames  [bits-in $geseg]]
        set bgoseglist [list-take $bgosegnames  [bits-in $bgoseg]]

        set list {}

        for {set i $lowchannel} {$i<=$highchannel} {incr i} {
# ge spectrum names
           foreach adc [list-take $genames [bits-in $ge]] {
              foreach s $geseglist {
                 foreach q $qlist {lappend list ClusterGe$i.Ge$s.$adc.q$q}
              }
           }
# append GIR (Ge card)
           if {$ge != 0} then {
              foreach adc [list-take $girnames [bits-in $gir]] {
                 foreach q $qlist {lappend list ClusterGe$i.$adc.q$q}
              }
           }
# append  bgo spectrum names
           foreach adc [list-take $bgonames [bits-in $bgo]] {
              foreach s $bgoseglist {
                 lappend list ClusterBGO$i.BGO$s.$adc
              }
           }
# append GIR (BGO card)
           if {$bgo != 0} then {
              foreach adc [list-take $girnames [bits-in $gir]] {
                 foreach q $qlist {lappend list ClusterBGO$i.$adc.q$q}
              }
           }
# append  bgo sum channel spectrum names
           foreach adc [list-take $sumnames [bits-in $sum]] {
               if {$adc == "Pattern"} then {
                  lappend list ClusterBGO$i.BGOPattern.High
                  lappend list ClusterBGO$i.BGOPattern.Low
               }
               if {$adc == "Energy"} then {lappend list ClusterBGO$i.BGOSum.Energy}
               if {$adc == "TAC"} then {lappend list ClusterBGO$i.BGOSum.TAC}
           }
        }
        return $list
}

proc click-segment {v} {
   global spectrumlist
   global geseg
   set geseg $v
   if {$spectrumlist!={}} then {paint-histlist; repaint}
}

proc click-bgo-segment v {
   global bgoseg spectrumlist

   switch $v {
      0   {set bgoseg 262143}
      1   {set bgoseg 0}
      2   {make-SELframe}
      default {return}
   }

   catch {:modify DET-BGOSEL :value $bgoseg}
   if {$spectrumlist!={}} then {paint-histlist; repaint}
}


proc click-bgo-select v {
   global bgoseg spectrumlist

   set bgoseg $v
   switch $bgoseg {
      0       {:modify DET-BGOSEG :value 0}
      262143  {:modify DET-BGOSEG :value 1}
      default {:modify DET-BGOSEG :value 2}
   }

   if {$spectrumlist!={}} then {paint-histlist; repaint}
}


proc range-of n {

        set fields [split $n "."]
        if {[string match "Cluster*" [first $fields]]} then {
                switch -glob [second $fields] {
                       GIR      {return 0x10000}
                       BGOPattern {return 0x400}
                       default  {return 0x2000}
                }
        }
}
