#    hist task code overlay for detector type "gocce"


set lowchannel  0
set highchannel 9
set maxchannel  9

set qualifier 1

set goccenames   {Energy Time}
set girnames    {GIR.EvNum}

set qnames   {q0 q1 q2 q3}
set qualnames {0 1 2 3}

set gocce 0
set gir 0

set xtalnames {A B C D}
set xtal 15
set segnames {0 1 2 3}
set seg 15

proc make-overlaypanel {} {
   global goccenames girnames
   global segnames seg xtalnames xtal qnames

      :layout V; :row-gap 10; :col-gap 20

      :message DET-M1   :label "GOCCE overlay applied"

      :number DET-LOW  :label "Low channel:"  :w 5 :valuex 15c
      :number DET-HIGH :label "High channel:" :w 5 :valuex 15c

      :choice DET-XTAL  :label "Crystal:"       :valuex 15c :exclusive F :strings $xtalnames :value $xtal
      :choice DET-SEG   :label "Segment:"       :valuex 15c :exclusive F :strings $segnames :value $seg

      :layout H
      :hskip 10
      :choice DET-GIR  :label "GIR" :exclusive F :strings [list EvNum] :value 0
      :next-row T
      :choice DET-QUAL :label "Qualifier"  :valuex 15c :exclusive F :strings $qnames :value 1
      :next-row T
      :choice DET-GOCCE   :label "Items"   :valuex 15c  :exclusive F :strings $goccenames  :value 0
}



#    this proc handles events from widgets created in the proc make-overlaypanel
#    The widgets DET-LOW; DET-HIGH and DET-QUAL are handled in the main click procedure
#    and are not required here

proc click-overlay {w v} {

   switch -glob $w {
      DET-GOCCE       {click-adc $v gocce}
      DET-SEG          {click-segment $v}
      DET-XTAL          {click-crystal $v}
      DET-GIR          {click-adc $v gir}
      default      {puts stdout $arg}
   }
}

proc repaint_overlay {} {
   global spectrumlist
   global gocce gir seg xtal qualifier

   if {[llength $spectrumlist] > 0} then {
       :modify DET-GOCCE    :value 0
       :modify DET-SEG      :value 0
       :modify DET-XTAL     :value 0
       :modify DET-QUAL     :value 0
       :modify DET-GIR :value 0
   } else {
       :modify DET-GOCCE   :value $gocce
       :modify DET-SEG      :value $seg
       :modify DET-XTAL     :value $xtal
       :modify DET-QUAL     :value $qualifier
       :modify DET-GIR      :value $gir
   }
}


proc get-spectrum-list {} {
        global spectrumlist
        global goccenames gocce girnames gir
        global lowchannel highchannel
        global qualifier qualnames
        global seg segnames
        global xtal xtalnames

        if {$spectrumlist != {}} then {return $spectrumlist};  # return names selected from spectrum list

        set qlist   [list-take $qualnames [bits-in $qualifier]]
        set seglist [list-take $segnames  [bits-in $seg]]
        set xtallist [list-take $xtalnames [bits-in $xtal]]

        set list {}

        for {set i $lowchannel} {$i<=$highchannel} {incr i} {
# append  spectrum names

            foreach x $xtallist {
            foreach s $seglist {
            foreach adc [list-take $goccenames  [bits-in $gocce]]  {
                 foreach q $qlist {lappend list GOCCE$i.Xtal$x.Seg$s.$adc.q$q}
            } } }
# append GIR
            foreach adc [list-take $girnames [bits-in $gir]] {
                 foreach q $qlist {lappend list GOCCE$i.$adc.q$q}
            }
        }
        return $list
}

proc click-segment {v} {
   global spectrumlist
   global seg
   set seg $v
   if {$spectrumlist!={}} then {paint-histlist; repaint}
}

proc click-crystal {v} {
   global spectrumlist
   global xtal
   set xtal $v
   if {$spectrumlist!={}} then {paint-histlist; repaint}
}

proc range-of n {

    set fields [split $n "."]
    if {[string match "GOCCE*" [first $fields]]} then {
        switch -glob [second $fields]  {
            GIR       {return 0x10000}
        }
        switch -glob [fourth $fields]  {
            Time       {return 4096}
            default    {return 16384}
        }
     }
}
