#    hist task code overlay for VXI card type "TDR32V"

set lowchannel 1
set highchannel 15
set maxchannel 255

set qualifier 1

set lowitem 0
set highitem 31
set maxitem 31

set itemmenu  {0-0 0-1 0-2 1 2 3 4 8 16 32 Reset}
set qnames    {q0 q2}
set qualnames {0 2}

if {![info exists XlatePres]}       {set XlatePres 0}
if {![info exists Xlate]}           {set Xlate(zzz) zzz}
if {![info exists XlateR]}          {set XlateR(zzz) zzz}
if {![info exists usetdr32vnames]}  {set usetdr32vnames 0}
if {![info exists skiptdr32vnames]} {set skiptdr32vnames 0}

proc make-Xlate {n} {
   global env Xlate XlateR XlatePres
   
   unset Xlate
   unset XlateR
   set Xlate(zzz) zzz
   set XlateR(zzz) zzz
   set XlatePres 0

     set z [catch {set directory [file join $env(EXPERIMENTBASE) Histconfigs]} m]
     if {$z != 0} then {
        midas-warning "$m\nYou should probably Join an experiment!"
        close-frame
        return
     }

     if {![file exists $directory]} then {
        set z [catch {:file mkdir $directory} m]
        if {$z!=0} then {midas-warning "Cannot create directory $directory\n $m"}
#        :file chmod 777 $directoryopen
     }

   #  read name translation file

   set z [catch {open [file join $directory [set n]_names] r} f]
   if {$z != 0} then {
       set-footer "Name translation file [file join $directory [set n]_names] does not exist"
       set Names {}
   } else {
       clear-footer ""
       set Names [split [read $f] \n]
       close $f
   }
   
   foreach item $Names {
       if {$item == ""} {continue}
       if {[third $item] == 0} {continue}
       set Xlate([first $item])  [second $item]
       set XlateR([second $item])  [first $item]
       set XlatePres 1
   }
}

proc Give_Xlate {n} {translate $n}
proc Give_XlateR {n} {translateR $n}

make-Xlate tdr32v

proc translate {n} {
   global usetdr32vnames skiptdr32vnames
   global XlatePres Xlate

   if {$XlatePres == 0} {return $n}
   if {$usetdr32vnames == 0} {return $n}
   
   if {[info exists Xlate($n)]} {return $Xlate($n)}
   
   if {$skiptdr32vnames == 0} {return $n}
   return ""
}

proc translateR {n} {
   global XlateR

   if {[info exists XlateR($n)]} {return $XlateR($n)} else {return $n}
}

proc show-xlation {} {
    global Xlate XlateR

    set contents ""
    set l1 [lsort -dictionary [array names Xlate]]
    set l2 [lsort -dictionary [array names XlateR]]
    set i 0
    foreach name $l1 {
        if {$name == "zzz"} break
        set n1 [lindex $l1 $i]
        set n2 [lindex $l2 $i]
        
        set m [second [split [first [split $n1 .]] #]]
        set a [string range [second [split $n1 .]] 3 end]
        set N1 [expr ($m << 5) + $a]
        
        set m [second [split [first [split $XlateR($n2) .]] #]]
        set a [string range [second [split $XlateR($n2) .]] 3 end]
        set N2 [expr ($m << 5) + $a]
        
        append contents [format "%8d %24s %24s   %8s   %24s %24s %8d\n" $N1 $n1 $Xlate($n1) | $n2 $XlateR($n2) $N2]
        incr i
    }
    
    midas-information "$contents"
}

proc midas-information {text} {
   global _frame

    set F [self frame-name]
    set frame [self frame-name].info

    :frame $frame :label  "Information from [self frame-title]" :toplevel T
         wm protocol $frame WM_DELETE_WINDOW "destroy $frame"

    :galley G :deaf T :strings $text :font fixed :cols 132
    :show T
    
    set _frame $F
}

proc make-overlaypanel {} {
   global maxchannel maxitem rangemenu itemmenu qnames
   global env XlatePres usetdr32vnames skiptdr32vnames
   global _frame
   
     set _frame [self frame-name]

    :layout V; :row-gap 10; :col-gap 0; :row-gap 10

      :message DET-M1  :label "TDR32V module overlay applied"

      :number DET-LOW  :label "Low card:"  :min 0 :max $maxchannel :w 5 :valuex 15c
      :number DET-HIGH :label "High card:" :min 0 :max $maxchannel :w 5 :valuex 15c

#      :choice DET-QUAL :label "Qualifier"  :valuex 15c :exclusive F :strings $qnames   :value 1

      :number DET-LOWI  :label "Low adc:"  :min 0 :max $maxitem :w 5 :valuex 15c
      :number DET-HIGHI :label "High adc:" :min 0 :max $maxitem :w 5 :valuex 15c

      :hskip 40
      :menubutton DET-ITEM-MENU :label "adcs" :strings $itemmenu

      if {$XlatePres} {
          :checkbox DET-NAMES :label "Apply name translation on create"    :value $usetdr32vnames  :valuex 36c
          :checkbox DET-SKIP  :label "Skip unknown translations on create" :value $skiptdr32vnames :valuex 36c
          :button   DET-SHOW  :label "Show name translations" :valuex 36
      }
}


#    this proc handles events from widgets created in the proc make-overlaypanel
#    The widgets DET-LOW; DET-HIGH and DET-QUAL are handled in the main click procedure
#    and are not required here

proc click-overlay {w v} {
   global usetdr32vnames skiptdr32vnames

   switch -glob $w {
      DET-LOWI           {click-low-item $v}
      DET-HIGHI          {click-high-item $v}
      DET-ITEM-MENU      {click-item-menu $v}
      DET-NAMES          {set usetdr32vnames $v}
      DET-SKIP           {set skiptdr32vnames $v}
      DET-SHOW           {show-xlation}
      default            {}
   }
}

proc click-low-item v {
   global lowitem highitem maxitem
   set lowitem $v
   if {$lowitem<0} then {set lowitem 0}
   if {$lowitem>$maxitem} then {set lowitem $maxitem}
   if {$highitem<$lowitem} then {set highitem $lowitem}
   repaint
}

proc click-high-item v {
   global lowitem highitem maxitem
   set highitem $v
   if {$highitem<0} then {set highitem 0}
   if {$highitem>$maxitem} then {set highitem $maxitem}
   if {$lowitem>$highitem} then {set lowitem $highitem}
   repaint
}

proc click-item-menu v {
    global lowitem highitem maxitem
    switch $v  {
            Reset {
                 set highitem 0
                 set lowitem 0
            }
            0-0 {
                 set highitem 0
                 set lowitem 0
            }
            0-1 {
                 set highitem 1
                 set lowitem 0
            }
            0-2 {
                 set highitem 2
                 set lowitem 0
            }
            default {
                 set z [scan $v "%d" n]
                 if {$z==1} then {
                     if {[expr $lowitem+$n-1]==$highitem} then \
                        {incr lowitem $n; incr highitem $n} \
                     else {set highitem [expr $lowitem+$n-1]}
                 }
            }
    }
    if {$lowitem<0} then {set lowitem 0}
    if {$lowitem>$maxitem} then {set lowitem $maxitem}
    if {$highitem>$maxitem} then {set highitem $maxitem}
    if {$highitem<$lowitem} then {set highitem $lowitem}
    repaint
}

proc repaint_overlay {} {
   global lowitem highitem rangeofitem

   :modify DET-LOWI  :value $lowitem
   :modify DET-HIGHI :value $highitem
}

proc get-spectrum-list {} {
        global spectrumlist
        global qualifier qualnames
        global lowchannel highchannel lowitem highitem

        if {$spectrumlist != {}} then {return $spectrumlist};  # return names selected from spectrum list

#        set qlist [list-take $qualnames [bits-in $qualifier]]
        set list {}

        for {set i $lowchannel} {$i<=$highchannel} {incr i} {
# append spectrum names
           for {set j $lowitem} {$j<=$highitem} {incr j} {
              set name [translate tdr32v#$i.adc$j]
              if {$name != ""} then {lappend list $name}
           }
        }

       return $list
}

proc range-of n {
  global rangeofitem
  return [expr 16 * 1024]
}
