# qdc6414v.tcl---Setup frame for QDC6414 card

#     created 2/2/99  from QDC6414V card frame   VP

set debug 0
set started 0

set frame QDC6414V
set frametitle "QDC6414V Card Setup"
set icontitle QDC6414V

set chanlist {???}
set chanindex 0

set defaultchanlist {0 1 2 3 4 5 6 7}

set cardtype QDC6414V
set VXIGlobal ""

set presetslist {}


#    The array "table" which is indexed by the "widget name" contains all the information needed
#    for the register/display item
#
#    Each element of "table" is a list
#
#       0.    name of global parameter used to hold current value of the register
#       1.    procedure to be used to convert the widget value into parameter value
#       2.    procedure to be used to convert the parameter value into a widget value
#       3.    name of the register
#       4.    procedure to be used to convert the register value into parameter value
#       5.    procedure to be used to convert the parameter value into a register value
#       6.    name of unit used for register access
#       7.    widget type to be used for "by parameter" displays
#       8-10  parameters for the procedure creating "by parameter" displays
#              eg  if (7) is :slider then 8 is min value, 9 is max value and 10 is unit
#       11    for startup only - initial value of the parameter (1)
#       12    for startup only - inserted for this widget into the table "parameternames"


set table(CHANNEL) {channel widget->channel channel->widget nil
        nil nil nil nil nil nil nil 1 nil}

for {set i 0} {$i < 64} {incr i} {
   set table(ENABLE$i) "enable$i int-widget->param param->int-widget \"c$i.Enable\"
       int-reg->param param->int-reg nil nil nil nil nil 0 \"Enable$i\""
}

for {set i 0} {$i < 8} {incr i} {
   set table(VXMSKVOIE$i) "vxmskvoie$i int-widget->param param->int-widget \"VXMSKVOIE$i\"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil"
}

set table(VXTEMP1) {vxtemp1 int-widget->param param->int-widget "VXTEMP1"
       int-reg->param param->int-reg nil nil nil nil nil 0 "Integration gate"}
set table(VXTEMP2) {vxtemp2 int-widget->param param->int-widget "VXTEMP2"
       int-reg->param param->int-reg nil nil nil nil nil 0 "Validation delay"}
set table(VXSEUIL) {vxseuil int-widget->param param->int-widget "VXSEUIL"
       int-reg->param param->int-reg nil nil nil nil nil 0 "Pedestal"}
set table(VXAMPTST) {vxamptst int-widget->param param->int-widget "VXAMPTST"
       int-reg->param param->int-reg nil nil nil nil nil 0 "Test amplitude"}

set table(VXETAT1) {vxetat1 str-widget->param param->hex-widget "VXETAT1"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(VXSYNCTST) {vxsynctst str-widget->param param->hex-widget "VXSYNCTST"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(VXINIT) {vxinit str-widget->param param->hex-widget "VXINIT"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(VXINHIBIT) {vxinhibit str-widget->param param->hex-widget "VXINHIBIT"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(VXODC) {vxodc str-widget->param param->hex-widget "VXODC"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(VXREVT) {vxrevt str-widget->param param->hex-widget "VXREVT"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(VXCODAGE) {vxcodage str-widget->param param->hex-widget "VXCODAGE"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(VXLECTURE) {vxlecture str-widget->param param->hex-widget "VXLECTURE"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}

#    VXI Configuration Registers

set table(MODELCODE) {vximodcode str-widget->param param->hex-widget "VXI.ModelCode"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(SERIALNUM) {vxiserialnum str-widget->param param->hex-widget "VXI.SerialNum"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(MODLEVEL) {vximodlevel str-widget->param param->hex-widget "VXI.ModLevel"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(MANID) {vximanid str-widget->param param->hex-widget "VXI.ManID"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(DEVICE) {vxidevice str-widget->param param->hex-widget "VXI.DevClass"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(ADDRSPACE) {vxiaddrspace str-widget->param param->hex-widget "VXI.AddrSpace"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(STATUS) {vxistatus str-widget->param param->hex-widget "VXI.Status"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(OFFSET) {vxioffset str-widget->param param->hex-widget "VXI.Offset"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(LASTCARD) {lastcard int-widget->param param->int-widget "LastCard"
       int-reg->param param->int-reg nil nil nil nil nil 1 nil}


#    register selection for parameter save function

set registersavepatterns {
   VXTEMP1 VXTEMP2 VXSEUIL 
}
for {set i 0} {$i < 64} {incr i} {lappend registersavepatterns ENABLE$i}




# inspection tables must be in same order as entries in the :choice-stacks
# this is achieved by using these tables when generating the choices
# index number returned maps directly to the signal number to be written to 
# the Signal Select register

set insp_table(MUX-AI1) {"Insp.A1" AI}
set insp_table(MUX-AI2) {"Insp.A2" AI}
set insp_table(MUX-DI1) {"Insp.D1" DI}
set insp_table(MUX-DI2) {"Insp.D2" DI}
set insp_table(MUX-VI)  {"Insp.V"  VI}

set insp_menu(MUX-AI1) analogueinsptable
set insp_menu(MUX-AI2) analogueinsptable
set insp_menu(MUX-DI1) digitalinsptable
set insp_menu(MUX-DI2) digitalinsptable
set insp_menu(MUX-VI)  voltageinsptable


set inspection_widgetnames {MUX-AI1 MUX-AI2 MUX-DI1 MUX-DI2 MUX-VI}
set inspection_regnames    {Insp.A1 Insp.A2 Insp.D1 Insp.D2 Insp.V}

#    the inspection line tables are in parallel
#    for channel select * implies value determined by adc channel
#    for parameter select * implies disconnect

set analogueinsptable {Disconnected
}
set AI-ChanSelect {0}
set AI-Param      {*}


set digitalinsptable {Disconnected
}

set DI-ChanSelect {0
}

set DI-Param      {*
}

set voltageinsptable {Disconnected
}
set VI-ChanSelect {0}
set VI-Param      {*}


set digitalinspmenu {Disconnected
}




loadsource gir
loadsource common.vxi


proc click arg {
   global debug
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]
   if {$c==":quit"} then {return [click-quit $f]}
   switch -glob $w {
      *REDISPLAY   {click-redisplay}
      ENGINEERING  {click-$v}

      CHANNEL     {click-channel $v}

      MUX-*       {click-inspection $w [lindex $arg 4]}

      default     {click-on $w $v}
   }
}

proc click-enable {v} {
  switch $v {
      0   {for {set i 0} {$i < 4} {incr i} {click-on VXMSKVOIE$i 0}}
      1   {for {set i 0} {$i < 4} {incr i} {click-on VXMSKVOIE$i 255}}
  }
  click-redisplay
}


set electronicsframe "[self frame-name].electronics"

proc make-frame1 {} {
  global env chanlist
  global analogueinsptable digitalinspmenu voltageinsptable stepmenu stepmode

  :frame [self frame-name] :label "[self frame-title]" :show-footer T :resize F \
      :icon-image "@$env(ICONHOME)/vxi_module" \
      :icon-name "[self icon-title]"

   :panel P1 :layout T :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 10

      :choice-stack CHANNEL :label "Current Channel:" :strings $chanlist :valuex 18c
      :next-row T
      :message ENABLEM  :label "ADC enable" :bold T
      :hskip 50
      :button ENABLE    :label "Enable All"  :command "click-enable 1"
      :button DISABLE   :label "Disable All" :command "click-enable 0"
      :next-row T
      :row-gap 0; :col-gap 0
      set j 0
      for {set i 0} {$i < 64} {incr i} {
         :checkbox ENABLE$i :label $i :valuex 3c 
         incr j
         if {$j == 8} then {:next-row T; set j 0}
      }
      
   :panel P3 :layout T :padx 10 
     :layout V; :row-gap 5
      :slider VXTEMP1  :label "Integration gate:" :min 0 :max 4095   :valuex 20c
      :slider VXTEMP2  :label "Validation delay:" :min 0 :max 4095   :valuex 20c
      :slider VXSEUIL  :label "Pedestal:"         :min 0 :max 4095   :valuex 20c

   :panel P4 :layout T :padx 10 :pady 10
      :layout H; :col-gap 10; :row-gap 10
      :menubutton MUX-AI1 :label "Analogue Inspection 1:" :w 22 :menu $analogueinsptable
      :message MUX-AI1-USE   :label ""
      :next-row T
      :menubutton MUX-AI2 :label "Analogue Inspection 2:" :w 22 :menu $analogueinsptable
      :message MUX-AI2-USE   :label ""
      :next-row T
      :menubutton MUX-DI1 :label "Digital Inspection 1:"  :w 22 :menu $digitalinspmenu
      :message MUX-DI1-USE   :label ""
      :next-row T
      :menubutton MUX-DI2 :label "Digital Inspection 2:"  :w 22 :menu $digitalinspmenu
      :message MUX-DI2-USE   :label ""
      :next-row T
      :menubutton MUX-VI  :label "Voltage Inspection:"    :w 22 :menu $voltageinsptable
      :message MUX-VI-USE   :label ""

   :panel P5 :layout B :padx 10 :after FOOTER
     :layout H; :col-gap 30; :row-gap 5
      :button REDISPLAY
      :menubutton ENGINEERING :label "Engineering"  :menu [list electronics software]
      :button HELP  
   
 :show T

}

proc make-frame2 {} {}


proc make-electronics {} {
  global env electronicsframe electronicsvisible

  :frame "$electronicsframe" :label "QDC6414V Electronics" :show-footer T :resize F :transient T

    :panel ELECTRONICSPANEL :layout T :padx 5 :pady 5
     :layout V; :row-gap 5

      :message ELM1       :label "VXI Configuration Registers" :bold T
      :text MANID         :label "Manufacturer ID:"    :w 6  :read-only T   :valuex 20c
      :text DEVICE        :label "Device Class:"       :w 6  :read-only T   :valuex 20c
      :text ADDRSPACE     :label "Address Space:"      :w 6  :read-only T   :valuex 20c
      :text MODELCODE     :label "Model Code:"         :w 6  :read-only T   :valuex 20c
      :text SERIALNUM     :label "Serial Number:"      :w 6  :read-only T   :valuex 20c
      :text MODLEVEL      :label "Modification Level:" :w 6  :read-only T   :valuex 20c
      :text STATUS        :label "Status Register"     :w 6  :read-only T   :valuex 20c
      :text OFFSET        :label "Offset Register"     :w 6  :read-only T   :valuex 20c
      :checkbox LASTCARD  :label "Last card"                 :deaf T   :valuex 20c
      :next-row T

      :message ELM2       :label "Board Configuration Registers" :bold T
      :text VXSYNCTST     :label VXSYNCTST   :w 8  :deaf T  :valuex 20c
      :text VXINIT        :label VXINIT      :w 8  :deaf T  :valuex 20c
      :text VXINHIBIT     :label VXINHIBIT   :w 8  :deaf T  :valuex 20c
      :text VXODC         :label VXODC       :w 8  :deaf T  :valuex 20c
      :text VXREVT        :label VXREVT      :w 8  :deaf T  :valuex 20c
      :text VXCODAGE      :label VXCODAGE    :w 8  :deaf T  :valuex 20c
      :text VXLECTURE     :label VXLECTURE   :w 8  :deaf T  :valuex 20c
      :text VXETAT1       :label VXETAT1     :w 8  :deaf T  :valuex 20c

      :slider VXAMPTST    :label "Test amplitude:"   :min 0 :max 4095 
  
      :show T
   set electronicsvisible 1
}  
