
#    tdr32v.tcl    control for TDR32V modules

set frame TDR32V
set title "TDR32V Card Setup"
set icon TDR32V

set chanlist {???}
set chanindex 0
set chansel 1

set seglist {}
for {set i 0} {$i < 32} {incr i} {lappend seglist Ch$i}

set segindex 0
set segsel 1

set allchannels "\[?,??\]"
set allsegments "Ch?,.Ch??"

set defaultchanlist {0 1 2 3 4 5 6 7}
set cardtype TDR32V
set chantype ""
set VXIGlobal ""


# These warning lines are a hack to display a warning for non-ADC mode

set adcwarning "***** Card operating as Pattern Register... No ADC conversion *****"

proc do-warning v {
    if {[info commands [:path ADCWARNING]] != ""} then {
        if {$v==3} then {
            global adcwarning
            :modify ADCWARNING :label $adcwarning
        } else {
            :modify ADCWARNING :label ""
        }
    }
}

proc param->int-widget-warning {p v u} {do-warning $v;return "$p :value $v" }


#    The array "table" which is indexed by the "widget name" contains all the information needed
#    for the register/display item
#
#    Each element of "table" is a list
#
#       0.    name of global parameter used to hold current value of the register
#       1.    procedure to be used to convert the widget value into parameter value
#       2.    procedure to be used to convert the parameter value into a widget value
#       3.    name of the register
#       4.    procedure to be used to convert the register value into parameter value
#       5.    procedure to be used to convert the parameter value into a register value
#       6.    name of unit used for register access
#       7.    widget type to be used for "by parameter" displays
#       8-10  parameters for the procedure creating "by parameter" displays
#              eg  if (7) is :slider then 8 is min value, 9 is max value and 10 is unit
#       11    for startup only - initial value of the parameter (1)
#       12    for startup only - inserted for this widget into the table "parameternames"


set table(CHANNEL) {channel widget->channel channel->widget nil
        nil nil nil nil nil nil nil 1 nil}
set table(ALLCHANS) {allchans int-widget->param param->int-widget nil nil nil nil nil nil nil nil 0 nil}

set table(ENABLE) {enable int-widget->param param->int-widget "ReadOutEnable"
       int-reg->param param->int-reg nil :checkbox-s nil nil nil 0 Enable}
       
set table(GATEWIDTH) {gatewidth slider-widget->param param->int-widget "GateWidth"
       unit-reg->param param->unit-reg ns :text-s 0 25600 1 1 "GateWidth"}

set table(CHMODE) {chmode int-widget->param param->int-widget "Group.Mode1.Gate"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}

set table(GRPAMODE0) {grpamode0 int-widget->param param->int-widget "Group.Mode1.bit4"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}

set table(GRPAMODE1) {grpamode1 int-widget->param param->int-widget "Group.Mode1.bit5"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}

set table(GRPAMODE2) {grpamode2 int-widget->param param->int-widget "Group.Mode1.bit6"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}

set table(GRPAMODE3) {grpamode3 int-widget->param param->int-widget "Group.Mode1.bit7"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}

set table(GRPBMODE0) {grpbmode0 int-widget->param param->int-widget "Group.Mode2.bit0"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}

set table(GRPBMODE1) {grpbmode1 int-widget->param param->int-widget "Group.Mode2.bit1"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}

set table(GRPBMODE2) {grpbmode2 int-widget->param param->int-widget "Group.Mode2.bit2"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}

set table(GRPBMODE3) {grpbmode3 int-widget->param param->int-widget "Group.Mode2.bit3"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}

set table(GRPBMODE4) {grpbmode4 int-widget->param param->int-widget "Group.Mode2.bit4"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}

set table(GRPBMODE5) {grpbmode5 int-widget->param param->int-widget "Group.Mode2.bit5"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}

set table(GRPBMODE6) {grpbmode6 int-widget->param param->int-widget "Group.Mode2.bit6"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}

set table(GRPBMODE7) {grpbmode7 int-widget->param param->int-widget "Group.Mode2.bit7"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}

set table(GRPCMODE) {grpcmode int-widget->param param->int-widget "Group.Mode3"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}

       
#    Card Control Register
set table(CCR) {ccr int-widget->param param->int-widget "CCR"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
       
set table(CCR0) {ccr0 int-widget->param param->int-widget "CCR.bit0"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(CCR1) {ccr1 int-widget->param param->int-widget "CCR.Scaler.Blocked.Enable"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(CCR2) {ccr2 int-widget->param param->int-widget "CCR.bit2"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(CCR3) {ccr3 int-widget->param param->int-widget "CCR.bit3"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(CCR4) {ccr4 int-widget->param param->int-widget "CCR.ADC.Go"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(CCR5) {ccr5 int-widget->param param->int-widget-warning "CCR.ADC.Mode"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(CCR8) {ccr8 int-widget->param param->int-widget "CCR.Reset"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
       
#    Card Status Register
set table(CSR) {csr int-widget->param param->int-widget "CSR"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
       
set table(CSR0) {csr0 int-widget->param param->int-widget "CSR.bit0"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(CSR1) {csr1 int-widget->param param->int-widget "CSR.bit1"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
       
#    Readout Status Register
set table(RSR) {rsr int-widget->param param->int-widget "Readout.Status"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
       
set table(RSR0) {rsr0 int-widget->param param->int-widget "RSR.bit0"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(RSR1) {rsr1 int-widget->param param->int-widget "RSR.bit1"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
       
#    Readout Control Register
set table(RCR) {rcr int-widget->param param->int-widget "Readout.Control"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
       
set table(RCR0) {rcr0 int-widget->param param->int-widget "RCR.bit0"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
       
#    Scalers
foreach i {0 1 2 3 4 5 6 7} {
set table(SCALER[set i]DATA)   "scaler[set i]data int-widget->param param->int-widget \"Scaler[set i].Data\"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil"
set table(SCALER[set i]GATE)   "scaler[set i]gate int-widget->param param->int-widget \"Scaler[set i].Gate\"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil"
set table(SCALER[set i]ENABLE) "scaler[set i]enable int-widget->param param->int-widget \"Scaler[set i].Enable\"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil"
}
set table(SCALERBLOCKEDDATA) {scalerblockeddata int-widget->param param->int-widget "Scaler.Blocked.Data"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
       
#    VXI Configuration Registers
set table(MODELCODE) {modcode str-widget->param param->hex-widget "VXI.ModelCode" 
   int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(SERIALNUM) {serialnum str-widget->param param->hex-widget "VXI.SerialNum" 
   int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(MODLEVEL) {modlevel str-widget->param param->hex-widget "VXI.ModLevel" 
   int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(MANID) {manid str-widget->param param->hex-widget "VXI.ManID" 
   int-reg->param param->int-reg nil nil nil nil nil 1 nil}
set table(OFFSET) {offset str-widget->param param->hex-widget "VXI.Offset" 
   int-reg->param param->int-reg nil nil nil nil nil 1 nil}
   
#     Module Control Register 
set table(LASTCARD) {lastcard int-widget->param param->int-widget "LastCard"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 nil}



#    register selection for parameter save function

set registersavepatterns {
    CCR Readout.Control  VETO_LUT_?
}
for {set i 0} {$i < 32} {incr i} {append registersavepatterns \
   " Ch$i.ReadOutEnable Ch$i.Group.Mode1 Ch$i.Group.Mode2 Ch$i.Group.Mode3 Ch$i.GateWidth"}
for {set i 0} {$i < 8}  {incr i} {append registersavepatterns " Scaler$i.Enable Scaler$i.Gate"}

# inspection tables must be in same order as entries in the :choice-stacks
# this is achieved by using these tables when generating the choices
# index number returned maps directly to the signal number to be written to 
# the Signal Select register

set insp_table(MUX-AI1) {"Insp.A1" AI}
set insp_table(MUX-AI2) {"Insp.A2" AI}
set insp_table(MUX-DI1) {"Insp.D1" DI}
set insp_table(MUX-DI2) {"Insp.D2" DI}
set insp_table(MUX-DI3) {"Insp.D3" DI}
set insp_table(MUX-DI4) {"Insp.D4" DI}

set insp_menu(MUX-AI1) analogueinsptable
set insp_menu(MUX-AI2) analogueinsptable
set insp_menu(MUX-DI1) digitalinsptable
set insp_menu(MUX-DI2) digitalinsptable
set insp_menu(MUX-DI3) digitalinsptable
set insp_menu(MUX-DI4) digitalinsptable


set inspection_widgetnames {MUX-AI1 MUX-AI2 MUX-DI1 MUX-DI2 MUX-DI3 MUX-DI4}
set inspection_regnames    {Insp.A1 Insp.A2 Insp.D1 Insp.D2 Insp.D3 Insp.D4}

#    the inspection line tables are in parallel
#    for channel select * implies value determined by adc channel
#    for parameter select * implies disconnect

set analogueinsptable {Disconnected
        Input PDS
}
set AI-ChanSelect {0 * *}
set AI-Param      {* 1 2}

set digitalinsptable {Disconnected
    Front_Panel_Gate_Input Blocked_Increment Timestamp_Counter_Clear Timestamp_Counter_Enable SYNC Sync_error_reset
    Sync_to_error Gate_inminus1 Gate_inminus2 Gate_inplus1 Gate_inplus2 Ready_for_next Wait Data_Ready FIFO_Read_request
    FIFO_Write_request FIFO_Full Pause Resume Halted Sync_Pulse Sync_A_not_equal_B Sync_error FIFO_empty Blocked_Scaler_Overflow
    Veto_0 Veto_1 Veto_2 Veto_3 Veto_4 Veto_5 Veto_6 Veto_7
    Gate_input_to_width_chip ADC_Gate ADC_Veto ADC_Selected ADC_Data_Available
    ADC_readout_fifo_empty ADC_readout_fifo_full ADC_readout_fifo_write 
    ADC_reset_after_conversion_and_readout_0 ADC_over_range_0 ADC_Veto_0
    ADC_reset_after_conversion_and_readout_1 ADC_over_range_1 ADC_Veto_1
    SHARC_clock SHARC_Ack SHARC_data0 SHARC_data1 SHARC_data2 SHARC_data3
    Ts_ready Ts_read Sharc_fifo_read_request Sharc_fifo_write_request Readout_Halted Readout_Anyfull Readout_Fifo_Empty
}


#    for CHIP
set DI-ChipSelect {0 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1
                     2 2 2 2 2 2 2 2 2 2 2
                     3 3 3 3 3 3 3 3 3 3 3
                     4 4 4 4 4 4 4 4 4 4 4 4 4
}

#    for Signal
set DI-Param      {* 0 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23
                     0 1 2 3 4 5 6 7 0 1 2
                     0 1 2 3 4 0 1 2 0 1 2
                     0 1 2 3 4 5 6 7 8 9 10 11 12
}

#    for Channel
set DI-Select     {* * 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32
                     32 32 32 32 32 32 32 32 * * * 
                     * * * * * 32 32 32 48 48 48
                     32 32 32 32 32 32 32 32 32 32 32 32
                  }

set digitalinspmenu {Disconnected
    Front_Panel_Gate_Input Blocked_Increment Timestamp_Counter_Clear Timestamp_Counter_Enable SYNC Sync_error_reset
    Sync_to_error Gate_inminus1 Gate_inminus2 Gate_inplus1 Gate_inplus2 Ready_for_next Wait Data_Ready FIFO_Read_request
    FIFO_Write_request FIFO_Full Pause Resume Halted Sync_Pulse Sync_A_not_equal_B Sync_error FIFO_empty Blocked_Scaler_Overflow
    {Front_Panel_Veto
    Veto_0 Veto_1 Veto_2 Veto_3 Veto_4 Veto_5 Veto_6 Veto_7
    } 
    {ADC
    Gate_input_to_width_chip ADC_Gate ADC_Veto ADC_Selected ADC_Data_Available
    ADC_readout_fifo_empty ADC_readout_fifo_full ADC_readout_fifo_write
    }
    {ADC_0=>15 
    ADC_reset_after_conversion_and_readout_0 ADC_over_range_0 ADC_Veto_0
    }
    {ADC_16=>31
    ADC_reset_after_conversion_and_readout_1 ADC_over_range_1 ADC_Veto_1
    }
    {SHARC
    SHARC_clock SHARC_Ack SHARC_data0 SHARC_data1 SHARC_data2 SHARC_data3
    }
    Ts_ready Ts_read Sharc_fifo_read_request Sharc_fifo_write_request Readout_Halted Readout_Anyfull Readout_Fifo_Empty
}

loadsource common.vxi

set parameter [first $widgetnames]
set register [first $regnames]

proc make-checkbox-panel-s {w} {
	global chanlist seglist
	foreach c $chanlist {
            set i 0
            foreach s $seglist {
	        :checkbox [set w]_[set c]_[set s] :valuex 50 :label "$c$s:"
                incr i
                if {$i == 16} {:next-row T}
	    }
            :next-row T
            :vskip 10
	}
}

proc make-text-panel-s {w width} {
	global chanlist seglist
	foreach c $chanlist {
           set i 0
           foreach s $seglist {
              :text [set w]_[set c]_[set s] :valuex 50 :label "$c$s:" :w 5
              incr i
              if {$i == 16} {:next-row T}
           }
           :next-row T
           :vskip 10
       }
}

proc click arg {
   global debug userselection allchans allsegs
   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]
   if {$c==":quit"} then {return [click-quit $f]}
   switch -glob $w {
      *REDISPLAY  {click-redisplay}
      FORMAT      {click-format}
      ENGINEERING {click-$v}
      EXPERT      {do_EXPERT-$v}
      
      CHANNEL     {click-channel $v}
      SEGMENT     {click-segment $v}
      ALLCHANS    {click-which click-whichchans $v}
      ALLSEGS     {click-which click-whichsegs $v}

      CHANSEL   {click-chanselection $v}
      SEGSEL    {click-segselection $v}
      
      MUX-*       {click-inspection $w $i}

      VETOLUT*     {click-vetolut [string range $w 7 end] $v}
      READOUT*     {click-readoutident [string range $w 7 end] $v}

      PARAM     {click-param $v}

      ENABLE_*   {
          if {$allsegs < 0 || $allchans < 0} then {set userselection 1}
             set c [second [split $w _]]
             set s [string range [third [split $w _]] 2 end]
             click-on ENABLE_[set c]_Ch[set s] $v
             set userselection 0
      }

      GATEWIDTH_*   {
          if {$allsegs < 0 || $allchans < 0} then {set userselection 1}
             set c [second [split $w _]]
             set s [string range [third [split $w _]] 2 end]
             click-on GATEWIDTH_[set c]_Ch[set s] $v
             set userselection 0
      }

      default   {
          if {$allsegs < 0 || $allchans < 0} then {set userselection 1}
             click-on $w $v
             set userselection 0
      }
   }
}


set electronicsframe [self frame-name].electronics
set electronicsvisible 0

set expertframe [self frame-name].expert
set expertvisible 0

set fifosframe [self frame-name].fifos
set fifosvisible 0

set vetolutframe [self frame-name].vetolut
set vetolutvisible 0

set readoutframe [self frame-name].readout
set readoutvisible 0

proc do_EXPERT-Setup {} {
   global expertvisible expertframe
   if {$expertvisible==0} then {make-expert} else {open-frame $expertframe}
   click-redisplay
}

proc do_EXPERT-FIFOs {} {
   global fifosvisible fifosframe
   if {$fifosvisible==0} then {make-fifos} else {open-frame $fifosframe}
   click-redisplay
}

proc do_EXPERT-VetoLUT {} {
   global vetolutvisible vetolutframe
   if {$vetolutvisible==0} then {make-vetolut} else {open-frame $vetolutframe}
   redisplay-vetolut
}

proc do_EXPERT-Readout {} {
   global readoutvisible readoutframe
   if {$readoutvisible==0} then {make-readout} else {open-frame $readoutframe}
   redisplay-readout
}

proc click-which {p v} {
   global allchans allsegs

   $p $v

   if {$allchans || $allsegs} then {set inact T} else {set inact F}
   :modify CHANRESET :inactive $inact
}

proc click-vetolut {i v} {
   global chanlist chanindex

   set c [lindex $chanlist $chanindex]

   if {$i < 64} then {
      set data [lreplace [read-reg $c "" VETO_LUT_1] [expr $i-0] [expr $i-0] $v]
      write-reg $c "" VETO_LUT_1 $data 0
      return
   }

   if {$i < 128} then {
      set data [lreplace [read-reg $c "" VETO_LUT_2] [expr $i-64] [expr $i-64] $v]
      write-reg $c "" VETO_LUT_2 $data 0
      return
   }

   if {$i < 192} then {
      set data [lreplace [read-reg $c "" VETO_LUT_3] [expr $i-128] [expr $i-128] $v]
      write-reg $c "" VETO_LUT_3 $data 0
      return
   } 

   if {$i < 256} then {
      set data [lreplace [read-reg $c "" VETO_LUT_4] [expr $i-192] [expr $i-192] $v]
      write-reg $c "" VETO_LUT_4 $data 0
      return
   } 
}

proc click-readoutident {i v} {
   global chanlist chanindex

   set c [lindex $chanlist $chanindex]

   if {$i < 64} then {

      set data [lreplace [read-reg $c "" Ident&Control_RAM_1] $i $i $v]
      write-reg $c "" Ident&Control_RAM_1 $data 0
   } else {
      set data [lreplace [read-reg $c "" Ident&Control_RAM_2] [expr $i-64] [expr $i-64] $v]
      write-reg $c "" Ident&Control_RAM_2 $data 0
   }
}

proc redisplay-vetolut {} {
    global chanlist chanindex

      set c [lindex $chanlist $chanindex]
      set data "[read-reg $c "" VETO_LUT_1] [read-reg $c "" VETO_LUT_2] [read-reg $c "" VETO_LUT_3] [read-reg $c "" VETO_LUT_4]"
      for {set i 0} {$i < 256} {incr i} {
          :modify VETOLUT[set i]  :value [lindex $data $i]
      }
}

proc redisplay-readout {} {
    global chanlist chanindex

      set c [lindex $chanlist $chanindex]
      redisplay-readoutevents
      set data "[read-reg $c "" Ident&Control_RAM_1] [read-reg $c "" Ident&Control_RAM_2]"
      for {set i 0} {$i < 128} {incr i} {
          :modify READOUT[set i]  :value [lindex $data $i]
      }
}

proc redisplay-readoutevents {} {
    global chanlist chanindex

      set c [lindex $chanlist $chanindex]
      :modify READOUTEVENTS :value [read-reg $c "" Readout.Events]
}

proc redisplay-readoutfifo {} {
    global chanlist chanindex

      set c [lindex $chanlist $chanindex]
      :modify READOUTFIFOH :value [read-reg $c "" Readout.FIFO.H]
      :modify READOUTFIFOM :value [read-reg $c "" Readout.FIFO.M]
      :modify READOUTFIFOL :value [read-reg $c "" Readout.FIFO.L]
}

proc redisplay-fifos {} {
   global chanlist chanindex

    set c [lindex $chanlist $chanindex]
    for {set i 0} {$i < 32} {incr i} {
        :modify FIFO$i :value [read-reg $c "" Ch$i.ADC_FIFO_Full]
    }

}

proc do-redisplay {} {
    global vetolutvisible readoutvisible fifosvisible
    click-redisplay
    if {$vetolutvisible == 1} then {redisplay-vetolut}
    if {$readoutvisible == 1} then {redisplay-readout}
    if {$fifosvisible == 1} then {redisplay-fifos}
}

proc do-special-reset {} {
    global chanlist chanindex
    global seglist segindex
    global EGreport

    set egr $EGreport
    set EGreport -1
    frame-busy

    set gm [read-reg [lindex $chanlist $chanindex] [lindex $seglist $segindex] Group.Mode1.Gate]
    set en [read-reg [lindex $chanlist $chanindex] [lindex $seglist $segindex] ReadOutEnable]
    write-reg [lindex $chanlist $chanindex] [lindex $seglist $segindex] Group.Mode1.Gate 0 0
    write-reg [lindex $chanlist $chanindex] [lindex $seglist $segindex] ReadOutEnable 0 0
    write-reg [lindex $chanlist $chanindex] [lindex $seglist $segindex] ReadOutEnable 1 0
    write-reg [lindex $chanlist $chanindex] [lindex $seglist $segindex] ReadOutEnable $en 0
    write-reg [lindex $chanlist $chanindex] [lindex $seglist $segindex] Group.Mode1.Gate $gm 0

    frame-idle
    set EGreport $egr
}

proc do-scalersclear {} {
    global chanlist chanindex

    write-reg [lindex $chanlist $chanindex] "" Scalers.Clear 0 0
    do-redisplay
}

proc do-scalersxfer {} {
    global chanlist chanindex

    write-reg [lindex $chanlist $chanindex] "" Scalers.Xfer 0 0 
    do-redisplay
}

proc do-scalerblockedclear {} {
    global chanlist chanindex

    write-reg [lindex $chanlist $chanindex] "" Scaler.Blocked.Clear 0 0
    do-redisplay
}


set chmodelist [list Disabled Direct Neighbour "Group 1 or More" "Group only 1"]

proc make-frame1 {} {
  global env chanlist seglist
  global digitalinspmenu analogueinsptable
  global chmodelist

  :frame [self frame-name] :label "[self frame-title]" :show-footer T :resize F \
      :icon-image "@$env(ICONHOME)/vxi_module" \
      :icon-name "[self icon-title]"

   :panel P1 :layout T :padx 10 :pady 5
     :layout H; :row-gap 10; :col-gap 10
      :choice-stack CHANNEL :label "Current Module:"   :strings $chanlist :valuex 16c
      :choice-stack SEGMENT :label "Current Channel:"  :strings $seglist  :valuex 16c
      :hskip 50
      :checkbox ENABLE :label "Enable:"
      :next-row T
      :choice ALLCHANS :label "Act on Modules?"   :valuex 16c :strings [list Current All Select] :value 0
      :choice ALLSEGS  :label "Act on Channels?"  :valuex 16c :strings [list Current All Select] :value 0

   :panel P2 :layout T :padx 10 :pady 10
     :layout H; :row-gap 5; :col-gap 2
      :slider GATEWIDTH    :label "Gate Width (ns):" :min 50 :max 25600   :valuex 20c
      :next-row T
      :message GATEM :label "Gate selection" :bold T
      :choice-stack CHMODE :label "Channel Mode"  :strings $chmodelist
      :hskip 90
      :button CHANRESET   :label "Channel \"Reset\""  :inactive F :command do-special-reset
      :next-row T
      :message MODEM1 :label "Neighbour: enable for channel" :valuex 32c
      :checkbox GRPAMODE0 :label "-2"
      :checkbox GRPAMODE1 :label "-1"
      :checkbox GRPAMODE2 :label "+1"
      :checkbox GRPAMODE3 :label "+2"
      :next-row T
      :message MODEM2 :label "Group 1 or More: enable for channel" 
      foreach i {0 1 2 3 4 5 6 7} {
      :checkbox GRPBMODE$i :label $i
      }
      :next-row T
      :message MODEM3 :label "Group only 1: Front Panel Input" :valuex 32c
      :choice-stack  GRPCMODE :label "" :strings [list 0 1 2 3 4 5 6 7]
      
      
   :panel P3 :layout T :padx 10
     :layout H
      :message SCALERM1 :label "Scalers" :bold T
      :hskip 15
      :message ADCWARNING :label "" :bold T 
      :next-row T
      :message SCALERM2 :label "Enable" :valuex 8c
      :message SCALERM3 :label "Gate"   :valuex 8c
      :message SCALERM4 :label "Value"  :valuex 12c
      :hskip 90
      :message SCALERM5 :label "Enable" :valuex 8c
      :message SCALERM6 :label "Gate"   :valuex 8c
      :message SCALERM7 :label "Value"  :valuex 12c
      :next-row T
      :row-gap 5; :col-gap 25
      foreach i {0 2 4 6} {
          :checkbox SCALER[set i]ENABLE  :label "[set i]"  :valuex 3c
          :text SCALER[set i]GATE :label ""  :w 2
          :text SCALER[set i]DATA :label ""  :w 10 :deaf T
          incr i
          :hskip 50
          :checkbox SCALER[set i]ENABLE  :label "[set i]"  :valuex 3c
          :text SCALER[set i]GATE :label ""   :w 2
          :text SCALER[set i]DATA :label ""   :w 10 :deaf T
          :next-row T
      }
      :text SCALERBLOCKEDDATA :label "Blocked Event Scaler" :valuex 20c :w 10 :deaf T
      :next-row T; :col-gap 10
      :button SCALERSXFER   :label "Transfer Scalers"   :command do-scalersxfer
      :button SCALERSCLEAR  :label "Clear Scalers 0=>7" :command do-scalersclear
      :button SCALERBLOCKEDCLEAR :label "Clear Blocked Scaler"  :command do-scalerblockedclear

   :panel P5 :layout T :padx 10 :pady 10
      :layout H; :col-gap 10; :row-gap 10
      :menubutton MUX-AI1 :label "Analogue Inspection 1:" :w 22 :menu $analogueinsptable
      :message MUX-AI1-USE   :label ""
      :next-row T
      :menubutton MUX-AI2 :label "Analogue Inspection 2:" :w 22 :menu $analogueinsptable
      :message MUX-AI2-USE   :label ""
      :next-row T
      :menubutton MUX-DI1 :label "Digital Inspection 1:"  :w 22 :menu $digitalinspmenu
      :message MUX-DI1-USE   :label ""
      :next-row T
      :menubutton MUX-DI2 :label "Digital Inspection 2:"  :w 22 :menu $digitalinspmenu
      :message MUX-DI2-USE   :label ""
      :next-row T
      :menubutton MUX-DI3 :label "Digital Inspection 3:"  :w 22 :menu $digitalinspmenu
      :message MUX-DI3-USE   :label ""
      :next-row T
      :menubutton MUX-DI4 :label "Digital Inspection 4:"  :w 22 :menu $digitalinspmenu
      :message MUX-DI4-USE   :label ""


   :panel P6 :layout B :padx 10 :after FOOTER
     :layout H; :col-gap 10
      :button REDISPLAY  :command do-redisplay
      :button FORMAT        :label "By Parameter"
      :menubutton EXPERT   :label "Expert" :menu [list Setup FIFOs "VetoLUT" Readout]
      :menubutton ENGINEERING :label "Engineering"  :menu [list electronics software]
      :button HELP  
   
 :show T
}

proc make-frame2 {} {
  global env parameternames

  :frame "[self frame-name]" :label "[self frame-title]" :show-footer T  :resize F \
      :icon-image "@$env(ICONHOME)/vxi_module" \
      :icon-name "[self icon-title]"

        :panel PARAMS :layout T :padx 5 :pady 5
         :layout H; :col-gap 10
          :choice-stack PARAM :label "Parameter:" :valuex 15c :strings $parameternames
          :hskip 70
          :choice ALLCHANS :label "Act on Modules?" :valuex 18c :strings [list Current All Select] :value 0
          :choice ALLSEGS  :label "Act on Channels?" :valuex 18c :strings [list Current All Select] :value 0

        :panel PANEL :layout T :padx 5

        :panel BUTTONS :layout B :padx 5 :after FOOTER
         :layout H
          :button REDISPLAY
          :button FORMAT       :label "By Channel"
          :hskip 80
          :button HELP     :command click-help

  :show T

}

proc make-electronics {} {
  global electronicsframe electronicsvisible

  :frame "$electronicsframe" :label "TDR32V Electronics" :resize F :transient T

   :panel P20 :layout T :padx 5 :pady 5
      :layout H; :row-gap 5
      :message VXIREGISTER       :label "VXI Registers:"  :bold T        :valuex 20c
    :next-row T
      :text MODELCODE     :label "Model Code:" :w 6  :read-only T        :valuex 20c
    :next-row T
      :text MODLEVEL      :label "Modification Level:" :w 6 :read-only T :valuex 20c
    :next-row T
      :text MANID         :label "Manufacturer ID:" :w 6  :read-only T   :valuex 20c
    :next-row T
      :text SERIALNUM     :label "Serial Number:" :w 6 :read-only T      :valuex 20c
    :next-row T
      :text OFFSET        :label "Offset:" :w 6 :read-only T      :valuex 20c

  :show T
 set electronicsvisible 1
}


proc make-expert {} {
  global expertframe expertvisible

  :frame "$expertframe" :label "TDR32V Card Setup (expert)" :resize F :transient T

   :panel P21 :layout T :padx 5 :pady 5
      :layout V; :row-gap 5
      
      :message MCCR  :label "Card Control Register" :bold T
      :checkbox CCR0  :label "Sync to error"           :valuex 20c
      :checkbox CCR1  :label "Enable Blocked Scaler"   :valuex 20c
      :checkbox CCR2  :label "Sync Error reset"        :valuex 20c
      :checkbox CCR3  :label "Timestamp Count Enable"  :valuex 20c
      :checkbox CCR4  :label "ADC Go"                  :valuex 20c
      :text CCR5      :label "ADC Mode"       :w 4     :valuex 20c
      :checkbox CCR8  :label "Reset"                   :valuex 20c
      
      :message MRCR  :label "Readout Control Register" :bold T
      :choice RCR0  :label "Readout Method"     :strings [list SHARC VME]  :valuex 20c :w 8
      
      :message MCSR  :label "Card Status Register" :bold T
      :checkbox CSR0  :label "Blocked Scaler Overflow"   :deaf T  :valuex 20c
      :checkbox CSR1  :label "Sync Error"  :deaf T :valuex 20c
      
      :message MRSR  :label "Readout Status Register" :bold T
      :checkbox RSR0  :label "Halted due to blocked readout"   :deaf T  :valuex 25c
      :checkbox RSR1  :label "Any ADC FIFO full"  :deaf T :valuex 25c
      
  :show T
 set expertvisible 1
}

proc make-fifos {} {
  global fifosframe fifosvisible

  :frame "$fifosframe" :label "TDR32V ADC FIFOs state" :resize F :transient T

   :panel P24 :layout T :padx 5 :pady 5
      :layout H; :row-gap 5; :col-gap 5

     :message FIFOS :label "ADC FIFO full" :bold T
     :next-row T

      for {set i 0} {$i < 32} {} {
          :checkbox FIFO$i :label $i :valuex 3c
          incr i
          if {[expr ($i / 8) * 8] == $i} then {:next-row T}
      }

     :button FIFOUPDATE :label "Update" :command redisplay-fifos
      
  :show T
 set fifosvisible 1
}

proc make-vetolut {} {
  global vetolutframe vetolutvisible

  :frame "$vetolutframe" :label "TDR32V Card Setup (VETO LUT)" :resize F :transient T

   :panel P22 :layout T :padx 5 :pady 5
      :layout H; :row-gap 5;  :col-gap 5
      
      for {set i 0} {$i < 256} {incr i} {
          :text VETOLUT[set i]  :label $i  :valuex 4c  :w 12
          set j [expr $i + 1]
          if {[expr ($j/8)*8] == $j} {:next-row T}
      }
      
  :show T
 set vetolutvisible 1
}

proc make-readout {} {
  global readoutframe readoutvisible

  :frame "$readoutframe" :label "TDR32V Card Setup (Readout)" :resize F :transient T

   :panel P23 :layout T :padx 5 :pady 5
      :layout H; :row-gap 5;  :col-gap 5
      
      :message READOUTM1 :label "Readout FIFO" :bold T
      :next-row T
      :text READOUTEVENTS :label "Events in readout FIFO"  :valuex 20c :w 6 :deaf T
      :hskip 50
      :button READOUTUPDATE :label "Update"  :command redisplay-readoutevents
      :next-row T
      :text READOUTFIFOH  :label "Readout FIFO" :w 12 :valuex 20c :deaf T
      :text READOUTFIFOM  :label ""             :w 12 :deaf T
      :text READOUTFIFOL  :label ""             :w 12 :deaf T
      :hskip 50
      :button READOUTNEXT :label "Next"  :command redisplay-readoutfifo
      :next-row T
      
      :message READOUTM2 :label "Readout Ident Store" :bold T
      :next-row T
      for {set i 0} {$i < 128} {incr i} {
          :text READOUT[set i]  :label $i  :valuex 4c  :w 12
          set j [expr $i + 1]
          if {[expr ($j/8)*8] == $j} {:next-row T}
      }
      
  :show T
 set readoutvisible 1
}

proc make-chanSELframe {} {
  global chansel chanlist

   :frame CHANSEL-FRAME :label "TDR32V Module Selection"  :show-footer F :resize F :transient T
   :panel CHANSEL-PANEL :layout T
   :choice CHANSEL :exclusive F :strings $chanlist :value $chansel
   :show T
}  

proc make-segSELframe {} {
  global segsel seglist

   :frame SEGSEL-FRAME :label "TDR32V Channel Selection"  :show-footer F :resize F :transient T
   :panel SEGSEL-PANEL :layout T
   :choice SEGSEL :exclusive F :strings $seglist :value $segsel
   :show T
} 


