#    vxisetup.tcl   -     VXI card setup frame
#       this implements a standard/novice interface   - experts look elsewhere

set debug 0
set started 0

set stage 1
set frame1exists 0

set frame VXISETUP
set frametitle "VXI Card Setup Frame"
set icontitle  "VXISetup"

#    load definition list
loadsource vxisetup.defn

set cardchoice {}
set tasknames    {}

foreach item $moduledefinition {
     append cardchoice " " \"[first $item]\"
     append tasknames    " " [second $item]
}

set cardindex 0
set card [lindex $cardchoice $cardindex]
set task [lindex $tasknames $cardindex]


#    inspection line configuration -  these are independent of card type
#    Note - these tables are NOT used by Eurogam modules

set insp_table(MUX-AI1) {"Insp.A1" AI}
set insp_table(MUX-AI2) {"Insp.A2" AI}
set insp_table(MUX-DI1) {"Insp.D1" DI}
set insp_table(MUX-DI2) {"Insp.D2" DI}

set insp_menu(MUX-AI1) analogueinsptable
set insp_menu(MUX-AI2) analogueinsptable
set insp_menu(MUX-DI1) digitalinsptable
set insp_menu(MUX-DI2) digitalinsptable

set inspection_widgetnames {MUX-AI1 MUX-AI2 MUX-DI1 MUX-DI2}
set inspection_regnames    {Insp.A1 Insp.A2 Insp.D1 Insp.D2}




proc click-default {w v i} {
   global debug allchans allsegs

   if {$debug} then {insert-debug "click-default called with $w $v $i"}

   switch -glob $w {

      CHANNEL           {click-channel $v}
      SEGMENT           {click-segment $v}

      NEXTCHAN          {click-next-channel 1}
      NEXTSEG           {click-next-segment 1}
      PREVCHAN          {click-next-channel -1}
      PREVSEG           {click-next-segment -1}

      ALLCHANS          {set allchans $v; update_chansandsegs}
      ALLSEGS           {set allsegs $v; update_chansandsegs}
      ALLCHANSANDSEGS   {set allsegs $v; set allchans $v; update_chansandsegs}

      MUX-*             {click-inspection $w $i}

      default           {click-on $w $v; if {$w == "TFA"} then {act_on_tfagain}}
   }
}

proc click-next-channel {v} {
    global chanlist chanindex

    incr chanindex $v
    if {$chanindex >= [llength $chanlist]} {set chanindex 0}
    if {$chanindex < 0} {set chanindex [expr [llength $chanlist] - 1]}
    click-channel $chanindex
}

proc click-next-segment {v} {
    global seglist segindex

    incr segindex $v
    if {$segindex >= [llength $seglist]} {set segindex 0}
    if {$segindex < 0} {set segindex [expr [llength $seglist] - 1]}     
    :modify NEXTSEG :inactive F
    click-segment $segindex
}

proc update_chansandsegs {} {
   global allchans allsegs segindex

   :modify ALLCHANS :value $allchans
   make-myserver
   if {$segindex < 0} then {return}
   if {$allchans == 1 && $allsegs == 1} then {
      :modify ALLCHANSANDSEGS :value 1
   } else {
      :modify ALLCHANSANDSEGS :value 0
   }
   :modify ALLSEGS :value $allsegs
}

proc click-card {i} {
   global info common.vxi.loaded
   global stage cardchoice tasknames cardindex card task

   if {[lindex $tasknames $i] == "triggermk2"} then {
       set-footer "Starting the Msster Trigger interface"
       after 200 clear-footer
       inform triggermk2 start
       return
   }

   clear_info

   set cardindex $i
   set card [lindex $cardchoice $cardindex]
   set task [lindex $tasknames $cardindex]

#    load hardware definition information for this card type
#    must remove certain components of previous cards

   global table
   if {[info exists table]} then {unset table}
   if {[loadsource vxisetup.$task.defn] == 0} then {
      :modify CARD :value -1
      midas-warning "Sorry - The definition file for this card is not available yet.\nUsing the expert interface.\n\n"
      set-footer "Starting the expert interface"
      after 200 clear-footer
      inform $task start
      return
   }

#    using the inspection line configuration table generate the 3 parallel tables required
#    for AI and DI

   global AI-Configuration analogueinsptable AI-ChanSelect AI-Param
   set analogueinsptable {}
   set AI-ChanSelect {}
   set AI-Param {}
   foreach item [set AI-Configuration] {
      lappend analogueinsptable [first $item]
      if {[llength $item] > 1} then {
         lappend AI-ChanSelect [second $item]
         lappend AI-Param [third $item]
      }
   }

   global DI-Configuration digitalinsptable DI-ChanSelect DI-Param
   set digitalinsptable {}
   set DI-ChanSelect {}
   set DI-Param {}
   foreach item [set DI-Configuration] {
      lappend digitalinsptable [first $item]
      if {[llength $item] > 1} then {
         lappend DI-ChanSelect [second $item]
         lappend DI-Param [third $item]
      }
   }

#   if {![info exists common.vxi.loaded]} then {loadsource common.vxi}
   loadsource common.vxi
   if {$stage == 1} then {set stage 2}
   if {$stage == 3} then {start-card}
}

proc click-choice3 {args} {
   global task
   inform $task start
}

#    choice 1 is the novice level interface
proc click-choice1 {args} {
   midas-warning "Sorry the novice interface is not available yet.\nPlease use the standard interface.\n\n"
}

#    choice 2 is the new standard level interface
proc click-choice2 {args} {
   global stage
   if {$stage == 1} then {
      midas-warning "Please choose VXI card type first"
   } else {
      set stage 3
      start-card
   }
}

proc show_info {w} {
   global info
   if {[info exists info($w)]} then {:modify DIALOG :label $info($w)}
}

proc clear_info {} {
   global info
   if {[info exists info]} then {unset info}
}


proc do_update {} {click-redisplay; update_chansandsegs}

proc empty_panel {which} {:modify $which :empty T}

proc click-expert {} {
   global tasknames cardindex
   set task [lindex $tasknames $cardindex]
   inform $task start
}


proc make-frame0 {} {
   global env stage cardindex
   global cardchoice task

  :frame [self frame-name] :label "[self frame-title]"     \
      :show-footer T  :resize F \
      :icon-image "@$env(ICONHOME)/vxi_crate" \
      :icon-name "[self icon-title]"

   :panel P1  :layout T :padx 10 :pady 10
      :layout V; :row-gap 10; :col-gap 10
      :choice-stack CARD :label "Choose VXI Card Type:" :strings $cardchoice :value -1

   :panel P2  :layout T :padx 10 :pady 10
      :layout H; :row-gap 10; :col-gap 10
      :button CHOICE1 :label "Start novice interface" :w 10  :h 4 :inactive T
      :button CHOICE2 :label "Start standard user interface"  :w 10  :h 4
      :button CHOICE3 :label "Start expert interface"  :w 10  :h 4

   :panel CLICK :layout T :padx 10
      :layout H; :row-gap 5; :col-gap 5
      :button HELP   :command click-help

   :show T
}


proc make-frame1 {} {
   global env stage frame1exists cardindex segindex frametitle
   global cardchoice card task
   global analogueinsptable digitalinsptable
   global chanlist seglist
   global table
   global channelitems segmentitems

   set frametitle "VXI $card Card Setup Frame"

   if {$frame1exists == 0} then {
      :frame [self frame-name] :label "[self frame-title]" :show-footer T  :resize F \
         :icon-image "@$env(ICONHOME)/vxi_crate" \
         :icon-name "[self icon-title]"
   } else {
      wm title . "[self frame-title]"
   }

   if {$frame1exists == 0} then {
      :panel P1  :layout T :padx 10 :pady 10 
        :choice-stack CARD :label "Choose VXI Card Type:" :strings $cardchoice :value $cardindex

      :panel DP :layout T :padx 5 :pady 0 :h 80 :w 250
      [:path DP] configure -relief raised -bd 1
   }

   if {$frame1exists == 0} then {
      :container C1 :layout L :padx 5 :pady 10 

      :panel CP1 :layout T :pady 10 
#      [:path CP1] configure -relief raised -bd 1
        :layout V; :row-gap 3; :col-gap 5

        :choice-stack CHANNEL :label "Current Channel:" :strings $chanlist
        :choice ALLCHANS :label "apply changes to all channels?" :valuex 50c \
            :strings [list no yes] :w 4
        :message XXX1 
   }

   if {$frame1exists == 0} then {
      :panel CP5 :layout T :h 220
#      [:path CP5] configure -relief raised -bd 1
   }

   :modify CP5 :empty T
   :layout V; :row-gap 5

   foreach item $channelitems {
      set label [lindex $table($item) 12]
      set type  [lindex $table($item) 7]
      set data1 [lindex $table($item) 8]
      set data2 [lindex $table($item) 9]
      switch $type {
        :choice-c  -
        :choice   {:choice   $item :label $label :valuex 25c :strings [list $data1 $data2] :value -1}
        :checkbox {:checkbox $item :label $label :valuex 25c :value 0}
        :slider-c  -
        :slider   {:slider   $item :label $label :valuex 25c :min $data1 :max $data2}
        default   {}
      }
   }

   if {$frame1exists == 0} then {
       :container S1 :layout R :padx 5 :pady 10 
       :panel SP1 :layout T :pady 10 
#       [:path SP1] configure -relief raised -bd 1
   } else {
       :modify SP1 :empty T
   }

      :layout V; :row-gap 3; :col-gap 5
   if {$segindex >= 0} then {
      :choice-stack SEGMENT :label "Current Segment:" :strings $seglist
      :choice ALLSEGS  :label "apply changes to all segments on the card?" \
          :valuex 50c :strings [list no yes] :w 4
      :choice ALLCHANSANDSEGS  :label "apply changes to all segments on all cards?" \
          :valuex 50c :strings [list no yes] :w 4
   } else {
      :message XXX2 :w 53
      :message XXX3 
      :message XXX4 
   }

   if {$frame1exists == 0} then {
      :panel SP5 :layout T :h 220
#      [:path SP5] configure -relief raised -bd 1
   }

   :modify SP5 :empty T
   :layout V; :row-gap 5
 
   foreach item $segmentitems {
      set label [lindex $table($item) 12]
      set type  [lindex $table($item) 7]
      set data1 [lindex $table($item) 8]
      set data2 [lindex $table($item) 9]
      switch $type {
         :choice-c  -
         :choice   {:choice   $item :label $label :valuex 25c :strings [list $data1 $data2] :value -1}
         :checkbox {:checkbox $item :label $label :valuex 25c :value 0}
         :slider-c  -
         :slider   {:slider   $item :label $label :valuex 25c :min $data1 :max $data2}
         default   {}
      }
   }

   if {$frame1exists == 0} then {
       :container C2 :layout B :padx 10 :after FOOTER
   }

   if {$frame1exists == 0} then {
      :panel P2A :layout R
#      [:path P2A] configure -relief raised -bd 1
   }

   :modify P2A :empty T
   :layout V; :row-gap 5; :col-gap 5

#    data readout panel
   :message DATAM :label "Readout" :bold T 
   global readoutitems
   foreach item $readoutitems {
       set label [lindex $table($item) 12]
       :choice $item :label $label :w 8 :valuex 12c :strings [list disable enable] :value -1
   }

   if {$frame1exists == 0} then {

      :panel P2B :layout L
#      [:path P2B] configure -relief raised -bd 1
      :layout H; :row-gap 8

#    inspection line panel
      :message MUXM  :label "Inspection Lines" :bold T
      :next-row T
      :menubutton MUX-AI1 :label "Analogue #1" :w 12
      :message MUX-AI1-USE   :label "                             "
      :next-row T
      :menubutton MUX-AI2 :label "Analogue #2" :w 12
      :message MUX-AI2-USE   :label ""
      :next-row T
      :menubutton MUX-DI1 :label "Digital #1"  :w 12
      :message MUX-DI1-USE   :label ""
      :next-row T
      :menubutton MUX-DI2 :label "Digital #2"  :w 12
      :message MUX-DI2-USE   :label ""

      :container end

   }
   :modify MUX-AI1 :menu $analogueinsptable
   :modify MUX-AI2 :menu $analogueinsptable
   :modify MUX-DI1 :menu $digitalinsptable
   :modify MUX-DI2 :menu $digitalinsptable

   if {$frame1exists == 0} then {
      :panel CLICK :layout B :padx 10 :pady 5 :after FOOTER
      :layout H; :row-gap 5; :col-gap 5
      :button UPDATE :label "update display" :command do_update
      :button NEXTCHAN  :label "channel+"
      :button NEXTSEG   :label "segment+"
      :button PREVCHAN  :label "channel-"
      :button PREVSEG   :label "segment-"
      :hskip 20
      :button CHECK  :label "Run VXI setup check" :inactive T
      :hskip 20
      :button EXPERT :label "Start Expert Interface"
      :button HELP   :command click-help
   }

   if {$segindex >= 0} then {
       :modify NEXTSEG :inactive F
       :modify PREVSEG :inactive F
   } else {
       :modify NEXTSEG :inactive T
       :modify PREVSEG :inactive T
   }

   if {$frame1exists == 0} then {:show T}

   :modify DP :empty T
   :message WARNING :colour "-fg red" :label " Settings on the left of the window apply to the whole VXI card; those on the right are unique to the channel or segment"
   :next-row T
   :message DIALOG

   set frame1exists 1
}



proc start-card {} {
   global stage
   global allchans allsegs segindex
   if {$stage != 3} then {return}
   set allchans 0
   if {$segindex >= 0} then {set allsegs 0}
   make-frame1
   EGbindings
   show_info CARD
   set-busy
   make-chanlist
   make-myserver
   do_update
   clear-busy
}

proc start args {
   global started stage
   if {$started == 0} then {
      make-frame0
      set stage 1
      set started 1
   } else {
      open-frame
      if {$stage == 3} then {
         set-busy
         make-chanlist
         make-myserver
         do_update
         clear-busy
      }
   }
}


#proc EG args  {insert-debug "EG $args"}
